from django.contrib.auth.views import LoginView, LogoutView
from django.urls import path

from . import views

urlpatterns = (
    path(
        'login/',
        LoginView.as_view(template_name="accounts/login.html"),
        name='accounts.login',
    ),
    path(
        'logout/',
        LogoutView.as_view(template_name="accounts/logout.html"),
        name='accounts.logout',
    ),
    path(
        'users/',
        views.users,
        name='accounts.user-list',
    ),
    path(
        'users/<str:username>/',
        views.user_detail,
        name='accounts.user-detail',
    ),
    path(
        'users/<str:username>/reactivate',
        views.user_activate,
        name='accounts.user-reactivate',
    ),
    path(
        'users/<str:username>/deactivate',
        views.user_activate,
        {'deactivate': True},
        name='accounts.user-deactivate',
    ),
    path(
        'settings/',
        views.user_settings,
        name='accounts.user-settings',
    ),
)



from django.conf import settings
from django.db import models
from django.db.models import TextChoices
from django.utils.translation import gettext_lazy as _


class AuditLogCategoryChoices(TextChoices):
    SECRET_READ = 'secret_read', _('secret_read')
    SECRET_ELEVATED_SUPERUSER_READ = 'secret_elevated_superuser_read', _('secret_elevated_superuser_read')
    SECRET_PERMISSION_VIOLATION = 'secret_permission_violation', _('secret_permission_violation')
    SECRET_CHANGED = 'secret_changed', _('secret_changed')
    SECRET_NEEDS_CHANGING_REMINDER = 'secret_needs_changing_reminder', _('secret_needs_changing_reminder')
    SECRET_SHARED = 'secret_shared', _('secret_shared')
    SECRET_SUPERUSER_SHARED = 'secret_superuser_shared', _('secret_superuser_shared')
    SECRET_SHARE_REMOVED = 'secret_share_removed', _('secret_share_removed')
    SECRET_SUPERUSER_SHARE_REMOVED = 'secret_superuser_share_removed', _('secret_superuser_share_removed')
    SECRET_ACCESS_REQUEST = 'secret_legacy_access_requests', _('secret_legacy_access_requests')

    USER_ACTIVATED = 'user_activated', _('user_activated')
    USER_DEACTIVATED = 'user_deactivated', _('user_deactivated')
    USER_SETTINGS_CHANGED = 'user_settings_changed', _('user_settings_changed')

    SHARE_AUTOMATICALLY_REVOKED = 'share_automatically_revoked', _('share_automatically_revoked')

    MISCELLANEOUS = 'miscellaneous', _('miscellaneous')


class LogEntry(models.Model):
    actor = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        models.PROTECT,
        blank=True,
        null=True,
        related_name='logged_actions',
    )
    category = models.CharField(
        choices=AuditLogCategoryChoices.choices,
        default=AuditLogCategoryChoices.MISCELLANEOUS,
        max_length=64,
    )
    group = models.ForeignKey(
        'auth.Group',
        models.PROTECT,
        blank=True,
        null=True,
        related_name='logged_actions',
    )
    message = models.TextField()
    reason = models.TextField(
        blank=True,
        null=True,
    )
    secret = models.ForeignKey(
        'secrets.Secret',
        models.PROTECT,
        blank=True,
        null=True,
        related_name='logged_actions',
    )
    secret_revision = models.ForeignKey(
        'secrets.SecretRevision',
        models.PROTECT,
        blank=True,
        null=True,
        related_name='logged_actions',
    )
    time = models.DateTimeField(
        auto_now_add=True,
    )
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        models.PROTECT,
        blank=True,
        null=True,
        related_name='affected_by_actions',
    )

    class Meta:
        indexes = [
            models.Index(fields=['category'], name='logentry_category_idx'),
            models.Index(fields=['time'], name='logentry_time_idx'),
            models.Index(fields=['category', 'time'], name='logentry_category_time_idx'),

            models.Index(fields=['actor'], name='logentry_actor_idx'),
            models.Index(fields=['group'], name='logentry_group_idx'),
            models.Index(fields=['secret'], name='logentry_secret_idx'),
            models.Index(fields=['user'], name='logentry_user_idx'),
        ]
        ordering = ('-time',)



# Generated by Django 4.2.1 on 2023-06-11 22:41

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("secrets", "0026_allowed_groups_users_intermediate"),
    ]

    operations = [
        migrations.AddConstraint(
            model_name="sharedsecretdata",
            constraint=models.CheckConstraint(
                check=models.Q(
                    models.Q(("group__isnull", False), ("user__isnull", True)),
                    models.Q(("group__isnull", True), ("user__isnull", False)),
                    _connector="OR",
                ),
                name="only_one_set",
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('auth', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='AccessRequest',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('closed', models.DateTimeField(null=True, blank=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('reason_request', models.TextField(null=True, blank=True)),
                ('reason_rejected', models.TextField(null=True, blank=True)),
                ('status', models.PositiveSmallIntegerField(default=1, choices=[(1, 'pending'), (2, 'rejected'), (3, 'approved')])),
                ('closed_by', models.ForeignKey(null=True, blank=True, on_delete=models.CASCADE, related_name='access_requests_closed', to=settings.AUTH_USER_MODEL)),
                ('requester', models.ForeignKey(on_delete=models.CASCADE, to=settings.AUTH_USER_MODEL, related_name='access_requests_created')),
                ('reviewers', models.ManyToManyField(to=settings.AUTH_USER_MODEL, related_name='access_requests_reviewed')),
            ],
            options={
                'ordering': ('-created',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Secret',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('access_policy', models.PositiveSmallIntegerField(default=1, choices=[(1, 'request'), (2, 'everyone'), (3, 'hidden')])),
                ('content_type', models.PositiveSmallIntegerField(default=1, choices=[(1, 'Password'), (2, 'Credit Card'), (3, 'File')])),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('description', models.TextField(null=True, blank=True)),
                ('last_read', models.DateTimeField(default=django.utils.timezone.now)),
                ('name', models.CharField(max_length=92)),
                ('needs_changing_on_leave', models.BooleanField(default=True)),
                ('status', models.PositiveSmallIntegerField(default=1, choices=[(1, 'OK'), (2, 'needs changing'), (3, 'deleted')])),
                ('url', models.URLField(null=True, blank=True)),
                ('username', models.CharField(null=True, max_length=255, blank=True)),
                ('allowed_groups', models.ManyToManyField(to='auth.Group', blank=True, related_name='allowed_passwords')),
                ('allowed_users', models.ManyToManyField(to=settings.AUTH_USER_MODEL, blank=True, related_name='allowed_passwords')),
                ('created_by', models.ForeignKey(on_delete=models.CASCADE, to=settings.AUTH_USER_MODEL, related_name='passwords_created')),
            ],
            options={
                'ordering': ('name',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='SecretRevision',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('encrypted_data', models.BinaryField()),
                ('length', models.PositiveIntegerField(default=0)),
                ('accessed_by', models.ManyToManyField(to=settings.AUTH_USER_MODEL)),
                ('secret', models.ForeignKey(on_delete=models.CASCADE, to='secrets.Secret')),
                ('set_by', models.ForeignKey(on_delete=models.CASCADE, to=settings.AUTH_USER_MODEL, related_name='password_revisions_set')),
            ],
            options={
                'ordering': ('-created',),
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='secretrevision',
            unique_together=set([('encrypted_data', 'secret')]),
        ),
        migrations.AddField(
            model_name='secret',
            name='current_revision',
            field=models.ForeignKey(null=True, blank=True, on_delete=models.CASCADE, related_name='_password_current_revision', to='secrets.SecretRevision'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='accessrequest',
            name='secret',
            field=models.ForeignKey(on_delete=models.CASCADE, to='secrets.Secret', related_name='access_requests'),
            preserve_default=True,
        ),
    ]



from django.contrib.postgres.search import SearchVector
from django.core.management.base import BaseCommand

from ...models import Secret


class Command(BaseCommand):
    help = 'Update search index'

    def handle(self, *args, **options):
        secrets_total = Secret.objects.count()
        Secret.objects.all().update(
            search_index=(
                SearchVector('name', weight='A') +
                SearchVector('description', weight='B') +
                SearchVector('username', weight='C') +
                SearchVector('filename', weight='D')
            )
        )
        self.stdout.write(self.style.SUCCESS(
            "Finished updating search index for {} objects.".format(secrets_total)
        ))



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-13 15:44
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('secrets', '0013_auto_20161021_1411'),
    ]

    operations = [
        migrations.AlterField(
            model_name='accessrequest',
            name='closed_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='access_requests_closed', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='accessrequest',
            name='requester',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='access_requests_created', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='accessrequest',
            name='secret',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='access_requests', to='secrets.Secret'),
        ),
        migrations.AlterField(
            model_name='secret',
            name='created_by',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='passwords_created', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='secret',
            name='current_revision',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='_password_current_revision', to='secrets.SecretRevision'),
        ),
        migrations.AlterField(
            model_name='secretrevision',
            name='secret',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='secrets.Secret'),
        ),
        migrations.AlterField(
            model_name='secretrevision',
            name='set_by',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='password_revisions_set', to=settings.AUTH_USER_MODEL),
        ),
    ]



import logging

from django.conf import settings
from django_auth_ldap.backend import LDAPBackend, _LDAPUser
from django_auth_ldap.config import LDAPSearch


logger = logging.getLogger(__name__)


def find_ldap_username_for_social_auth(details, *_args, **kwargs):
    if not kwargs.get('is_new'):
        return {}

    connection = _LDAPUser(LDAPBackend(), username='').connection
    ldap_mail_attribute = settings.AUTH_LDAP_USER_ATTR_MAP['email']
    social_auth_mail_value = details['email']
    logger.info(f'Trying to find LDAP username for social auth user {social_auth_mail_value}...')
    search = LDAPSearch(
        settings.AUTH_LDAP_USER_SEARCH.base_dn,
        settings.AUTH_LDAP_USER_SEARCH.scope,
        f'({ldap_mail_attribute}={social_auth_mail_value})',
        ['uid']
    )
    results = search.execute(connection)
    if results is not None and len(results) > 0:
        uid = results[0][1]['uid'][0]
        logger.info(f'Found LDAP username for social auth user {social_auth_mail_value}: {uid}')
        return {'username': uid}
    logger.info(f'No LDAP username found for social auth user {social_auth_mail_value}')
    return {}


def populate_from_ldap(*_args, **kwargs):
    LDAPBackend().populate_user(kwargs['user'].username)



# Generated by Django 4.2.1 on 2023-06-12 00:09

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("secrets", "0029_sharedsecretdata_granted_by"),
    ]

    operations = [
        migrations.AddField(
            model_name="sharedsecretdata",
            name="granted_on",
            field=models.DateTimeField(auto_now_add=True, null=True),
        ),
    ]



from logging import getLogger

from .models import LogEntry

AUDIT_LOG = getLogger(__name__)


def log(
    msg,
    level='info',
    category=None,
    actor=None,
    reason=None,
    secret=None,
    secret_revision=None,
    group=None,
    user=None,
):
    getattr(AUDIT_LOG, level)(msg)
    entry = LogEntry()
    entry.message = msg
    entry.category = category
    entry.actor = actor
    entry.reason = reason
    entry.secret = secret
    entry.secret_revision = secret_revision
    entry.group = group
    entry.user = user
    entry.save()



# Generated by Django 4.2.13 on 2024-07-15 11:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('secrets', '0032_alter_sharedsecretdata_granted_by'),
    ]

    operations = [
        migrations.AddField(
            model_name='secretrevision',
            name='encrypted_otp_key',
            field=models.BinaryField(blank=True, null=True),
        ),
    ]



# Generated by Django 4.2.5 on 2024-02-13 15:40

from django.db import migrations, models


def add_current_share_reasons_to_audit_log(apps, schema_editor):
    log_entry_model = apps.get_model('audit', 'LogEntry')
    shared_secret_data_model = apps.get_model('secrets', 'SharedSecretData')
    shares = shared_secret_data_model.objects.all().exclude(granted_by__isnull=True)
    shared_log_entries = log_entry_model.objects.filter(
        category__in=['secret_shared', 'secret_superuser_shared']
    ).only('secret__id', 'actor__id', 'reason')

    for share in shares:
        matched_entry = shared_log_entries.filter(
            actor__id=share.granted_by.id,
            secret__id=share.secret.id,
        ).order_by('time').last()

        # Matching can fail for immediate shares while creating new secrets
        if matched_entry:
            matched_entry.reason = share.grant_description
            matched_entry.save(update_fields=['reason'])


class Migration(migrations.Migration):
    dependencies = [
        ("audit", "0005_alter_logentry_category"),
        ("secrets", "0032_alter_sharedsecretdata_granted_by")
    ]

    operations = [
        migrations.AddField(
            model_name="logentry",
            name="reason",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.RunPython(
            code=add_current_share_reasons_to_audit_log,
            reverse_code=migrations.RunPython.noop,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-13 15:44
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('audit', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='logentry',
            name='actor',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='logged_actions', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='logentry',
            name='group',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='logged_actions', to='auth.Group'),
        ),
        migrations.AlterField(
            model_name='logentry',
            name='secret',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='logged_actions', to='secrets.Secret'),
        ),
        migrations.AlterField(
            model_name='logentry',
            name='secret_revision',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='logged_actions', to='secrets.SecretRevision'),
        ),
        migrations.AlterField(
            model_name='logentry',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='affected_by_actions', to=settings.AUTH_USER_MODEL),
        ),
    ]



# Generated by Django 4.2.1 on 2023-06-15 17:48

from django.db import migrations


def remove_inactivate_users_from_groups(apps, schema_editor):
    user_model = apps.get_model('auth', 'User')
    for user in user_model.objects.all().exclude(groups__isnull=False, is_active=True):
        user.groups.clear()


class Migration(migrations.Migration):
    dependencies = [('auth', '__latest__')]

    operations = [
        migrations.RunPython(remove_inactivate_users_from_groups),
    ]



from base64 import b64decode, b64encode
from configparser import ConfigParser
from gettext import gettext as _
from hashlib import sha1
from os import environ, umask
from os.path import exists, isfile
from random import choice
from string import ascii_letters, digits, punctuation
from urllib.parse import urlparse

from cryptography.fernet import Fernet
from django.core.exceptions import ImproperlyConfigured
from django.db.utils import ProgrammingError


class UnconfiguredSettingsError(Exception):
    def __str__(self):
        return _(
            "missing config file at {} (set env var TEAMVAULT_CONFIG_FILE to use a different path)"
        ).format(environ['TEAMVAULT_CONFIG_FILE'])


def configure_base_url(config, settings):
    settings.BASE_URL = config.get("teamvault", "base_url")
    settings.ALLOWED_HOSTS = [urlparse(settings.BASE_URL).hostname]


def configure_database(config):
    """
    Called directly from the Django settings module.
    """
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.postgresql',
            'HOST': get_from_config(config, "database", "host", "localhost"),
            'NAME': get_from_config(config, "database", "name", "teamvault"),
            'PASSWORD': get_from_config(config, "database", "password", ""),
            'PORT': get_from_config(config, "database", "port", "5432"),
            'USER': get_from_config(config, "database", "user", "teamvault"),
        },
    }
    return DATABASES


def configure_debugging(config, settings):
    enabled = get_from_config(config, "teamvault", "insecure_debug_mode", "no")
    if enabled.lower() in ("1", "enabled", "true", "yes"):
        settings.DEBUG = True
        settings.EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'
    else:
        settings.DEBUG = False


def configure_django_secret_key(config):
    """
    Called directly from the Django settings module.
    """
    return b64decode(config.get("django", "secret_key").encode()).decode('utf-8')


def configure_hashid(config):
    """
    Called directly from the Django settings module.
    """
    return (
        int(get_from_config(config, "hashid", "min_length", "6")),
        b64decode(config.get("hashid", "salt").encode()).decode('utf-8'),
    )


def configure_password_generator(config, settings):
    settings.PASSWORD_LENGTH = int(get_from_config(config, "password_generator", "length", 16))
    settings.PASSWORD_DIGITS = int(get_from_config(config, "password_generator", "digits", 2))
    settings.PASSWORD_UPPER = int(get_from_config(config, "password_generator", "upper", 2))
    settings.PASSWORD_LOWER = int(get_from_config(config, "password_generator", "lower", 2))
    settings.PASSWORD_SPECIAL = int(get_from_config(config, "password_generator", "special", 2))

    char_sum = settings.PASSWORD_SPECIAL + settings.PASSWORD_LOWER + settings.PASSWORD_UPPER + settings.PASSWORD_DIGITS

    if char_sum > settings.PASSWORD_LENGTH:
        raise ImproperlyConfigured(_(
            'The sum of characters defined in password settings exceeds the value set in password_length setting'
        ))


def configure_google_auth(config, settings):
    if not config.has_section("auth_google"):
        settings.GOOGLE_AUTH_ENABLED = False
        return

    settings.GOOGLE_AUTH_ENABLED = True

    settings.AUTHENTICATION_BACKENDS.insert(
        0,
        'social_core.backends.google.GoogleOAuth2',
    )

    settings.SOCIAL_AUTH_PIPELINE = [
        # Get the information we can about the user and return it in a simple
        # format to create the user instance later. In some cases the details are
        # already part of the auth response from the provider, but sometimes this
        # could hit a provider API.
        'social_core.pipeline.social_auth.social_details',

        # Get the social uid from whichever service we're authing thru. The uid is
        # the unique identifier of the given user in the provider.
        'social_core.pipeline.social_auth.social_uid',

        # Verifies that the current auth process is valid within the current
        # project, this is where emails and domains whitelists are applied (if
        # defined).
        'social_core.pipeline.social_auth.auth_allowed',

        # Checks if the current social-account is already associated in the site.
        'social_core.pipeline.social_auth.social_user',

        # Associates the current social details with another user account with
        # a similar email address.
        'social_core.pipeline.social_auth.associate_by_email',

        # Create a user account if we haven't found one yet.
        # Also see comment below for LDAP compatibility
        'social_core.pipeline.user.create_user',

        # Create the record that associates the social account with the user.
        'social_core.pipeline.social_auth.associate_user',

        # Populate the extra_data field in the social record with the values
        # specified by settings (and the default ones like access_token, etc).
        'social_core.pipeline.social_auth.load_extra_data',

        # Update the user record with any changed info from the auth service.
        'social_core.pipeline.user.user_details',
    ]

    settings.SOCIAL_AUTH_GOOGLE_OAUTH2_WHITELISTED_DOMAINS = [
        domain.strip() for domain in
        config.get("auth_google", "allowed_domains").split(",")
    ]

    settings.GOOGLE_AUTH_AVATARS = get_from_config(config, "auth_google", "use_avatars", True)
    if settings.GOOGLE_AUTH_AVATARS:
        settings.SOCIAL_AUTH_PIPELINE.append('teamvault.apps.accounts.utils.save_google_avatar')
    else:
        settings.SOCIAL_AUTH_PIPELINE.append('teamvault.apps.accounts.utils.save_gravatar')

    settings.SOCIAL_AUTH_GOOGLE_OAUTH2_KEY = config.get("auth_google", "oauth2_key")
    settings.SOCIAL_AUTH_GOOGLE_OAUTH2_SECRET = config.get("auth_google", "oauth2_secret")

    # LDAP compatibility settings
    if config.has_section("auth_ldap"):
        settings.SOCIAL_AUTH_PIPELINE.append('teamvault.apps.accounts.auth.populate_from_ldap')

        if get_from_config(config, "auth_google", "use_ldap_usernames", False):
            # Social Auth tries to create users with usernames by stripping the domain part of their email address.
            # This behaviour clashes when these usernames differ from the ones in a configured LDAP directory.
            # We cannot use "SOCIAL_AUTH_CLEAN_USERNAME_FUNCTION" here, because Social Auth only provides the username
            # of our chosen storage provider and not the full email address.
            settings.SOCIAL_AUTH_PIPELINE.insert(
                # Username fix has to happen before "social_core.pipeline.user.create_user".
                settings.SOCIAL_AUTH_PIPELINE.index('social_core.pipeline.user.create_user'),
                'teamvault.apps.accounts.auth.find_ldap_username_for_social_auth'
            )


def configure_huey(config):
    # For now, all tasks use the same crontab value. They're all
    # background maintenance task at the moment.
    freq = get_from_config(config, "tasks", "scheduler_frequency", "daily")
    if freq == "daily":
        scheduler_frequency = {
            'day': '*/1',
            'hour': '0',
            'minute': '0',
        }
    elif freq == "hourly":
        scheduler_frequency = {
            'hour': '*/1',
            'minute': '0',
        }
    elif freq == "minutely":
        scheduler_frequency = {
            'minute': '*/1',
        }
    else:
        raise RuntimeError(_(
            "Unknown value {freq} for task.scheduler_frequency in {path}"
        ).format(
            freq=freq,
            path=environ['TEAMVAULT_CONFIG_FILE'],
        ))

    revoke = get_from_config(
        config,
        "tasks",
        "revoke_unused_shares_after_days",
        None,
    )
    if not revoke is None:
        try:
            revoke = int(revoke)
        except ValueError:
            raise RuntimeError(_(
                "task.revoke_unused_shares_after_days must be an integer in {path}"
            ).format(
                path=environ['TEAMVAULT_CONFIG_FILE'],
            ))

    return {
        'revoke_unused_shares_after_days': revoke,
        'scheduler_frequency': scheduler_frequency,
    }


def configure_ldap_auth(config, settings):
    if not config.has_section("auth_ldap"):
        settings.LDAP_AUTH_ENABLED = False
        return

    settings.LDAP_AUTH_ENABLED = True

    from django_auth_ldap.config import LDAPSearch, MemberDNGroupType
    from ldap import (
        SCOPE_SUBTREE,
        OPT_X_TLS_CERTFILE,
        OPT_X_TLS_KEYFILE,
        OPT_X_TLS_REQUIRE_CERT,
        OPT_X_TLS_NEVER,
        OPT_X_TLS_NEWCTX,
    )

    settings.AUTHENTICATION_BACKENDS.insert(
        0,
        'django_auth_ldap.backend.LDAPBackend',
    )
    settings.AUTH_LDAP_SERVER_URI = config.get("auth_ldap", "server_uri")
    settings.AUTH_LDAP_BIND_DN = config.get("auth_ldap", "bind_dn")
    settings.AUTH_LDAP_BIND_PASSWORD = config.get("auth_ldap", "password")

    settings.AUTH_LDAP_USER_SEARCH = LDAPSearch(
        config.get("auth_ldap", "user_base_dn"),
        SCOPE_SUBTREE,
        get_from_config(config, "auth_ldap", "user_search_filter", "(cn=%(user)s)"),
    )
    settings.AUTH_LDAP_GROUP_SEARCH = LDAPSearch(
        config.get("auth_ldap", "group_base_dn"),
        SCOPE_SUBTREE,
        get_from_config(config, "auth_ldap", "group_search_filter", "(objectClass=group)"),
    )

    settings.AUTH_LDAP_GROUP_TYPE = MemberDNGroupType('member')
    settings.AUTH_LDAP_REQUIRE_GROUP = get_from_config(config, "auth_ldap", "require_group", None)

    settings.AUTH_LDAP_USER_ATTR_MAP = {
        "email": get_from_config(config, "auth_ldap", "attr_email", "mail"),
        "first_name": get_from_config(config, "auth_ldap", "attr_first_name", "givenName"),
        "last_name": get_from_config(config, "auth_ldap", "attr_last_name", "sn"),
    }
    settings.AUTH_LDAP_USER_FLAGS_BY_GROUP = {
        "is_staff": config.get("auth_ldap", "admin_group"),
        "is_superuser": config.get("auth_ldap", "admin_group"),
    }

    settings.AUTH_LDAP_ALWAYS_UPDATE_USER = True
    settings.AUTH_LDAP_FIND_GROUP_PERMS = False
    settings.AUTH_LDAP_MIRROR_GROUPS = True
    settings.AUTH_LDAP_CACHE_GROUPS = True
    settings.AUTH_LDAP_GROUP_CACHE_TIMEOUT = 900

    settings.AUTH_LDAP_CONNECTION_OPTIONS = {}
    settings.AUTH_LDAP_GLOBAL_OPTIONS = {}

    if get_from_config(config, "auth_ldap", "start_tls", "no").lower() in \
            ("1", "enabled", "true", "yes"):
        settings.AUTH_LDAP_START_TLS = True

    if get_from_config(config, "auth_ldap", "client_cert", None):
        settings.AUTH_LDAP_CONNECTION_OPTIONS[OPT_X_TLS_CERTFILE] = \
            config.get("auth_ldap", "client_cert")
        settings.AUTH_LDAP_CONNECTION_OPTIONS[OPT_X_TLS_KEYFILE] = \
            config.get("auth_ldap", "client_key")

    if get_from_config(config, "auth_ldap", "disable_server_cert_validation", "no").lower() in \
            ("1", "enabled", "true", "yes"):
        settings.AUTH_LDAP_CONNECTION_OPTIONS[OPT_X_TLS_REQUIRE_CERT] = \
            OPT_X_TLS_NEVER
        # must be set last, relies on dict insertion order
        settings.AUTH_LDAP_CONNECTION_OPTIONS[OPT_X_TLS_NEWCTX] = 0


def configure_logging(config):
    level = 'INFO'

    if get_from_config(config, "teamvault", "insecure_debug_mode", "no").lower() in \
            ("1", "enabled", "true", "yes"):
        level = 'DEBUG'

    LOGGING = {
        'version': 1,
        'disable_existing_loggers': False,
        'formatters': {
            'console': {
                'format': "[%(asctime)s] %(levelname)s %(module)s: %(message)s",
            },
        },
        'handlers': {
            'console': {
                'formatter': 'console',
                'level': 'DEBUG',
                'class': 'logging.StreamHandler',
            },
        },
        'loggers': {
            'django': {
                'handlers': ['console'],
                'level': 'INFO',
            },
            'django_auth_ldap': {
                'handlers': ['console'],
                'level': level,
            },
            'teamvault': {
                'handlers': ['console'],
                'level': level,
            },
        },
    }

    return LOGGING


def configure_max_file_size(config, settings):
    settings.TEAMVAULT_MAX_FILE_SIZE = int(
        get_from_config(config, "teamvault", "max_file_size", "5242880")
    )
    settings.FILE_UPLOAD_MAX_MEMORY_SIZE = settings.TEAMVAULT_MAX_FILE_SIZE


def configure_password_update_alert(config, settings):
    equivalent_true_values = ["1", "true", "enabled", "yes"]

    password_update_alert_value = get_from_config(config, "teamvault", "password_update_alert_activated", False)
    settings.PASSWORD_UPDATE_ALERT_ACTIVATED = str(password_update_alert_value).lower() in equivalent_true_values


def configure_session(config):
    """
    Called directly from the Django settings module.
    """
    age = int(get_from_config(config, "teamvault", "session_cookie_age", "3600"))
    expire = get_from_config(config, "teamvault", "session_expire_at_browser_close", "True")
    secure = get_from_config(config, "teamvault", "session_cookie_secure", "False")

    if age <= 0:
        age = 3600
    expire = expire.lower() in ("1", "enabled", "true", "yes")
    secure = secure.lower() in ("1", "enabled", "true", "yes")

    return age, expire, secure


def configure_superuser_reads(config, settings):
    settings.ALLOW_SUPERUSER_READS = False
    if get_from_config(config, "teamvault", "allow_superuser_reads", "False").lower() in ("1", "enabled", "true", "yes"):
        settings.ALLOW_SUPERUSER_READS = True


def configure_teamvault_secret_key(config, settings):
    from .models import Setting

    key = config.get("teamvault", "fernet_key")

    try:
        checksum = Setting.get("fernet_key_hash", default=None)
    except ProgrammingError:  # db not populated
        pass
    else:
        key_hash = sha1(key.encode('utf-8')).hexdigest()

        if checksum is None:
            Setting.set("fernet_key_hash", key_hash)

        elif key_hash != checksum:
            raise RuntimeError(_(
                "secret in '{path}' does not match SHA1 hash in database ({hash})"
            ).format(
                hash=checksum,
                path=environ['TEAMVAULT_CONFIG_FILE'],
            ))

    settings.TEAMVAULT_SECRET_KEY = key


def configure_time_zone(config):
    return get_from_config(config, "teamvault", "time_zone", "UTC")


def configure_whitenoise(settings):
    if not settings.DEBUG:
        settings.STORAGES = {
            'staticfiles': {
                'BACKEND': 'whitenoise.storage.StaticFilesStorage'
            }
        }


def create_default_config(filename):
    if exists(filename):
        raise RuntimeError("not overwriting existing path {}".format(filename))
    SECRET_KEY = "".join(choice(ascii_letters + digits + punctuation) for i in range(50))
    HASHID_SALT = "".join(choice(ascii_letters + digits + punctuation) for i in range(50))
    config = """
[teamvault]
# Set this to the URL users use to reach the application
base_url = https://example.com
# This key has been generated for you, there is no need to change it
fernet_key = {teamvault_key}
# do not enable this in production
insecure_debug_mode = disabled
# file uploads larger than this number of bytes will have their connection reset
max_file_size = 5242880
session_cookie_age = 3600
session_expire_at_browser_close = True
session_cookie_secure = False
time_zone = UTC
# allow_superuser_reads = False

#[password_generator]
#length = 16
#digits = 2
#upper = 2
#lower = 2
#special = 2

[django]
# This key has been generated for you, there is no need to change it
secret_key = {django_key}

[database]
host = localhost
name = teamvault
user = teamvault
password = teamvault

[hashid]
min_length = 6
# This salt has been generated for you, there is no need to change it
salt = {hashid_salt}

#[auth_ldap]
#server_uri = ldaps://ldap.example.com
##start_tls = yes
##client_cert = /path/to/cert.crt
##client_key = /path/to/key.key
##disable_server_cert_validation = no
#bind_dn = cn=root,dc=example,dc=com
#password = ******************
#user_base_dn = ou=users,dc=example,dc=com
##user_search_filter = (cn=%%(user)s)
#group_base_dn = ou=groups,dc=example,dc=com
##group_search_filter = (objectClass=group)
##require_group = cn=employees,ou=groups,dc=example,dc=com
##attr_email = mail
##attr_first_name = givenName
##attr_last_name = sn
#admin_group = cn=admins,ou=groups,dc=example,dc=com

#[auth_google]
#allowed_domains = example.com, another.example.com
#oauth2_key = 123456789.apps.googleusercontent.com
#oauth2_secret = ******************
#use_avatars = True

#[tasks]
#scheduler_frequency = daily  # or hourly, minutely
#revoke_unused_shares_after_days = 365  # task disabled if unset
    """.format(
        django_key=b64encode(SECRET_KEY.encode('utf-8')).decode('utf-8'),
        hashid_salt=b64encode(HASHID_SALT.encode('utf-8')).decode('utf-8'),
        teamvault_key=Fernet.generate_key().decode('utf-8'),
    )
    old_umask = umask(7)
    try:
        with open(filename, 'wt') as f:
            f.write(config.strip())
    finally:
        umask(old_umask)


def get_config():
    if not isfile(environ['TEAMVAULT_CONFIG_FILE']):
        raise UnconfiguredSettingsError()

    with open(environ['TEAMVAULT_CONFIG_FILE']) as f:
        # ConfigParser.read() will not complain if it can't read the
        # file, so we need to read it once ourselves to get a proper IOError
        f.read()

    config = ConfigParser()
    config.read(environ['TEAMVAULT_CONFIG_FILE'])
    return config


def get_from_config(config, section, option, default):
    if config.has_option(section, option):
        return config.get(section, option)
    else:
        return default



# Generated by Django 4.2.3 on 2023-07-20 12:41

from django.db import migrations, models

category_choices = [
    ("secret_read", "secret_read"),
    ("secret_elevated_superuser_read", "secret_elevated_superuser_read"),
    ("secret_permission_violation", "secret_permission_violation"),
    ("secret_changed", "secret_changed"),
    ("secret_needs_changing_reminder", "secret_needs_changing_reminder"),
    ("secret_legacy_access_requests", "secret_legacy_access_requests"),
    ("secret_shared", "secret_shared"),
    ("secret_superuser_shared", "secret_superuser_shared"),
    ("user_activated", "user_activated"),
    ("user_deactivated", "user_deactivated"),
    ("user_settings_changed", "user_settings_changed"),
    ("miscellaneous", "miscellaneous"),
]


def categorize_log_entries(apps, schema_editor):
    log_entry_model = apps.get_model('audit', 'LogEntry')

    mapping = {
        r"^.* used superuser privileges to read '.*'$": "secret_superuser_read",
        r"^.* read '.*'$": "secret_read",
        r"^.* tried to access '.*' without permission$": "secret_permission_violation",
        r"^.* shared '.* with .*$": "secret_shared",
        r"^.* granted access to \w+ '.*'.*$": "secret_shared",
        r"^.* set a new secret for '.*' \([\w\d]+->[\w\d]+\)$": "secret_changed",
        r"^.* deleted '.*' \(\d+:\d+\)$": "secret_changed",
        r"^.* restore '.*' \(\d+:\d+\)$": "secret_changed",
        r"^.* has \w+ access request #\d+ for .*, ?\w* allowing access to '.*'$": "secret_legacy_access_requests",
        r"^secret '.*' needs changing because user '.*' was deactivated$": "secret_needs_changing_reminder",
        r"^.* reactivated .*": "user_activated$",
        r"^.* deactivated .*, \d+ secrets marked for changing": "user_deactivated$",
    }

    for regex, category in mapping.items():
        log_entry_model.objects.filter(message__regex=regex).update(category=category)


class Migration(migrations.Migration):
    dependencies = [
        ("audit", "0002_auto_20170313_1544"),
    ]

    operations = [
        migrations.AddField(
            model_name="logentry",
            name="category",
            field=models.CharField(
                choices=category_choices,
                default="miscellaneous",
                max_length=64,
            ),
        ),
        migrations.RunPython(
            code=categorize_log_entries,
        ),
        migrations.AlterField(
            model_name="logentry",
            name="category",
            field=models.CharField(
                choices=category_choices,
                max_length=64,
            ),
        ),
    ]



from os import environ

from django.core.wsgi import get_wsgi_application


environ.setdefault("DJANGO_SETTINGS_MODULE", "teamvault.settings")
environ.setdefault("TEAMVAULT_CONFIG_FILE", "/etc/teamvault.cfg")

application = get_wsgi_application()



from django.contrib import messages
from django.contrib.auth.decorators import user_passes_test, login_required
from django.contrib.auth.models import User
from django.db import transaction
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.urls import reverse, reverse_lazy
from django.utils.translation import gettext_lazy as _
from django.views.decorators.http import require_http_methods
from django.views.generic import DetailView, ListView, UpdateView

from .forms import UserProfileForm
from .models import UserProfile as UserProfileModel
from ..audit.auditlog import log
from ..audit.models import AuditLogCategoryChoices
from ..secrets.models import Secret, SecretRevision


class UserProfile(UpdateView):
    form_class = UserProfileForm
    model = UserProfileModel
    template_name = "accounts/user_settings.html"
    success_url = reverse_lazy('accounts.user-settings')

    def get_object(self, *args, **kwargs):
        return UserProfileModel.objects.get_or_create(user=self.request.user)[0]

    def form_valid(self, form):
        response = super().form_valid(form)
        messages.success(self.request, _('Successfully updated settings.'))
        return response


user_settings = login_required(UserProfile.as_view())


class UserList(ListView):
    context_object_name = 'users'
    paginate_by = 25
    template_name = "accounts/user_list.html"

    def get_queryset(self):
        return User.objects.order_by('username')


users = user_passes_test(lambda u: u.is_superuser)(UserList.as_view())


class UserDetail(DetailView):
    context_object_name = 'user'
    model = User
    slug_field = 'username'
    slug_url_kwarg = 'username'
    template_name = 'accounts/user_detail.html'


user_detail = user_passes_test(lambda u: u.is_superuser)(UserDetail.as_view())


@user_passes_test(lambda u: u.is_superuser)
@require_http_methods(["POST"])
def user_activate(request, username, deactivate=False):
    user = get_object_or_404(
        User,
        username=username,
        is_active=deactivate,
    )
    user.is_active = not deactivate
    user.save()
    if deactivate:
        user.groups.clear()
        accessed_revs = SecretRevision.objects.filter(
            accessed_by=user,
        ).exclude(
            secret__needs_changing_on_leave=False,
        ).exclude(
            secret__status=Secret.STATUS_NEEDS_CHANGING,
        ).select_related(
            'secret',
        )
        secrets = set()
        for rev in accessed_revs:
            if rev.is_current_revision:
                secrets.add(rev.secret)
        with transaction.atomic():
            for secret in secrets:
                secret.status = Secret.STATUS_NEEDS_CHANGING
                secret.save()
                msg = _(
                    "secret '{secret}' needs changing because user '{user}' was deactivated"
                ).format(
                    secret=secret.name,
                    user=user.username,
                )
                log(
                    msg,
                    actor=request.user,
                    category=AuditLogCategoryChoices.USER_DEACTIVATED,
                    secret=secret,
                    user=user,
                )
        log(
            _("{actor} deactivated {user}, {secrets} secrets marked for changing").format(
                actor=request.user.username,
                user=user.username,
                secrets=len(secrets),
            ),
            actor=request.user,
            category=AuditLogCategoryChoices.USER_DEACTIVATED,
            user=user,
        )
    else:
        log(
            _("{actor} reactivated {user}").format(
                actor=request.user.username,
                user=user.username,
            ),
            actor=request.user,
            category=AuditLogCategoryChoices.USER_ACTIVATED,
            user=user,
        )
    return HttpResponseRedirect(reverse('accounts.user-detail', kwargs={'username': user.username}))



# Generated by Django 4.2.16 on 2024-11-14 13:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("audit", "0007_alter_logentry_category"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="logentry",
            index=models.Index(fields=["category"], name="logentry_category_idx"),
        ),
        migrations.AddIndex(
            model_name="logentry",
            index=models.Index(fields=["time"], name="logentry_time_idx"),
        ),
        migrations.AddIndex(
            model_name="logentry",
            index=models.Index(fields=["category", "time"], name="logentry_category_time_idx"),
        ),
        migrations.AddIndex(
            model_name="logentry",
            index=models.Index(fields=["actor"], name="logentry_actor_idx"),
        ),
        migrations.AddIndex(
            model_name="logentry",
            index=models.Index(fields=["group"], name="logentry_group_idx"),
        ),
        migrations.AddIndex(
            model_name="logentry",
            index=models.Index(fields=["secret"], name="logentry_secret_idx"),
        ),
        migrations.AddIndex(
            model_name="logentry",
            index=models.Index(fields=["user"], name="logentry_user_idx"),
        ),
    ]



from os.path import join


def configure_webpack(settings):
    settings.WEBPACK_LOADER = {
        'DEFAULT': {
            'CACHE': not settings.DEBUG,
            'BUNDLE_DIR_NAME': 'bundled/',  # must end with slash
            'STATS_FILE': join(settings.PROJECT_ROOT, 'webpack-stats.json'),
            'POLL_INTERVAL': 0.1,
            'TIMEOUT': None,
            'IGNORE': [r'.+\.hot-update.js', r'.+\.map'],
            'LOADER_CLASS': 'webpack_loader.loader.WebpackLoader',
        }
    }



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-30 10:51
from __future__ import unicode_literals

import django.contrib.postgres.search
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('secrets', '0011_remove_secret_search_index'),
    ]

    operations = [
        migrations.AddField(
            model_name='secret',
            name='search_index',
            field=django.contrib.postgres.search.SearchVectorField(blank=True, null=True),
        ),
    ]



from django.shortcuts import render
from django.views.generic.base import ContextMixin


def handler404(request, exception, **kwargs):
    if request.user.is_authenticated:
        return render(request, "404_loggedin.html", status=404)
    else:
        return render(request, "404_anon.html", status=404)


class FilterMixin(ContextMixin):
    _bound_filter = None
    filter_class = None
    request = None

    def get_filter(self, queryset):
        if self.filter_class is None:
            raise AttributeError('No filter class specified when using FilterMixin!')

        self._bound_filter = self.filter_class(self.request.GET, queryset)
        return self._bound_filter

    def get_filtered_queryset(self, queryset):
        return self.get_filter(queryset=queryset).qs

    @staticmethod
    def manipulate_filter_form(bound_data, filter_form):
        """
        Can be overwritten in subclasses to add custom behaviour for a single view
        Has to return a filter_form
        """
        return filter_form

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        bound_filter_data = self._bound_filter.form.cleaned_data
        new_filter_form = self._bound_filter.get_form_class()()

        active_filters = {}
        initial = {}

        for field, field_data in bound_filter_data.items():
            if field_data:
                field_label = new_filter_form.fields[field].label
                initial[field] = field_data
                if isinstance(field_data, (tuple, list)):
                    active_filters[field_label] = field_data
                else:
                    # filter_items needs to be a list for us to iterate over
                    active_filters[field_label] = [field_data]

        new_filter_form.initial = initial
        new_filter_form = self.manipulate_filter_form(bound_filter_data, new_filter_form)
        context.update({
            'active_filters': active_filters,
            'filter_form': new_filter_form,
        })
        return context



# Generated by Django 3.2.6 on 2021-08-24 12:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('secrets', '0023_auto_20190822_1234'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='secret',
            name='notify_on_access_request',
        ),
        migrations.AlterField(
            model_name='secret',
            name='access_policy',
            field=models.PositiveSmallIntegerField(choices=[(1, 'discoverable'), (2, 'everyone'), (3, 'hidden')], default=1),
        ),
        migrations.DeleteModel(
            name='AccessRequest',
        ),
    ]



from datetime import date

from django import forms
from django.contrib.auth.models import Group, User
from django.core.exceptions import ValidationError
from django.forms.widgets import RadioSelect
from django.utils.translation import gettext_lazy as _

from .models import Secret, SharedSecretData
from .utils import extract_url_and_params
from .validators import is_valid_b32_string

GENERIC_FIELDS_HEADER = ['name']
GENERIC_FIELDS_FOOTER = [
    'access_policy',
    'description',
    'grant_description',
    'needs_changing_on_leave',
    'shared_groups_on_create',
]


class SecretForm(forms.ModelForm):
    access_policy = forms.ChoiceField(
        choices=Secret.ACCESS_POLICY_CHOICES,
        initial=Secret.ACCESS_POLICY_DISCOVERABLE,
        widget=RadioSelect(),
    )
    description = forms.CharField(
        required=False,
        widget=forms.Textarea(attrs={'cols': '15', 'rows': '4'})
    )
    needs_changing_on_leave = forms.BooleanField(
        help_text=_("This secret will be marked as 'needs changing' when a user who accessed it is deactivated."),
        initial=True,
        label=_('Needs changing'),
        required=False,
        widget=forms.CheckboxInput(attrs={'class': 'form-check-input'})
    )
    shared_groups_on_create = forms.ModelMultipleChoiceField(
        help_text=_('Default groups you configured in your settings will be selected automatically.'),
        label=_('Share with groups'),
        queryset=Group.objects.all().order_by('name'),
        required=False,
    )
    grant_description = forms.CharField(
        label=_('Reason'),
        required=False,  # see clean() below
        widget=forms.Textarea(attrs={'cols': '15', 'rows': '2'}),
    )

    def clean(self):
        cleaned_data = super().clean()
        if cleaned_data['shared_groups_on_create'] and not cleaned_data['grant_description']:
            self.add_error(
                'grant_description',
                _('Please provide a valid reason to share this secret with the groups selected above.')
            )


class CCForm(SecretForm):
    expiration_month = forms.IntegerField(
        min_value=1,
        max_value=12,
    )
    expiration_year = forms.IntegerField(
        min_value=date.today().year,
        max_value=date.today().year + 50,
    )
    holder = forms.CharField(
        label=_('Name on card'),
    )
    number = forms.CharField()
    password = forms.CharField(
        required=False,
        widget=forms.PasswordInput,
    )
    security_code = forms.CharField(
        label=_('Card CVV'),
        required=False,
        widget=forms.NumberInput,
    )

    class Meta:
        model = Secret
        fields = (
            GENERIC_FIELDS_HEADER +
            [
                'expiration_month',
                'expiration_year',
                'holder',
                'number',
                'password',
                'security_code',
            ] +
            GENERIC_FIELDS_FOOTER
        )


class FileForm(SecretForm):
    file = forms.FileField(
        # Note: There's also custom handling for files with a file size above the FILE_UPLOAD_MAX_MEMORY_SIZE limit
        #  in the corresponding view
        allow_empty_file=False,
        required=False,
    )

    class Meta:
        model = Secret
        fields = (
            GENERIC_FIELDS_HEADER +
            ['file'] +
            GENERIC_FIELDS_FOOTER
        )


class PasswordForm(SecretForm):
    otp_key = forms.CharField(
        required=False
    )
    otp_key_data = forms.CharField(
        required=False,
    )
    password = forms.CharField(
        required=False,
    )
    url = forms.CharField(
        max_length=255,
        required=False,
        label='URL',
    )
    username = forms.CharField(
        max_length=255,
        required=False,
    )

    def clean_password(self):
        if self.instance.pk is None and not self.cleaned_data['password']:
            # password is only required when adding a new secret
            raise forms.ValidationError(_("Please enter a password."))
        return self.cleaned_data['password']

    def clean_otp_key_data(self):
        cleaned_otp_key_data = self.cleaned_data['otp_key_data']
        if self.instance.pk is not None and not cleaned_otp_key_data:
            return cleaned_otp_key_data

        try:
            as_url, data_params = extract_url_and_params(cleaned_otp_key_data)
        except Exception:
            raise forms.ValidationError(_('OTP key should have a format like this: ___?secret=___&digits=___ ...'))
        secret = data_params['secret'][0] if 'secret' in data_params else ''

        is_valid_b32_string(secret)
        return cleaned_otp_key_data

    class Meta:
        model = Secret
        fields = (
            GENERIC_FIELDS_HEADER +
            ['password', 'username', 'url'] +
            GENERIC_FIELDS_FOOTER
        )


class SecretShareForm(forms.ModelForm):
    group = forms.ModelChoiceField(
        required=False,
        queryset=Group.objects.none(),  # will be set in view
    )

    user = forms.ModelChoiceField(
        required=False,
        queryset=User.objects.none(),  # will be set in view
    )

    grant_description = forms.CharField(
        label=_('Reason'),
        required=True,
        widget=forms.Textarea(attrs={'cols': '15', 'rows': '2'})
    )

    granted_until = forms.DateTimeField(
        required=False,
        widget=forms.DateTimeInput(),
    )

    def clean(self):
        cleaned_data = super().clean()
        if (cleaned_data['group'] and cleaned_data['user']) or (not cleaned_data['group'] and not cleaned_data['user']):
            raise ValidationError('Choose exactly one group *or* one user to share the secret with.')

    class Meta:
        fields = ['group', 'user', 'granted_until', 'grant_description']
        model = SharedSecretData



# Generated by Django 4.2.3 on 2023-08-29 12:45

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("accounts", "0003_usersettings_avatar"),
    ]

    operations = [
        migrations.AlterField(
            model_name="usersettings",
            name="hide_deleted_secrets",
            field=models.BooleanField(
                default=True,
                help_text="Hides deleted secrets per default. Enable them in filters to see them again.",
            ),
        ),
    ]



from datetime import timedelta
from hashlib import sha256
from json import JSONDecodeError, dumps, loads
from operator import itemgetter

from cryptography.fernet import Fernet
from django.conf import settings
from django.contrib.auth.models import Group, User
from django.contrib.postgres.search import SearchVector, SearchVectorField
from django.core.exceptions import PermissionDenied, ValidationError
from django.db import models
from django.db.models import BooleanField, Case, Max, Q, Value, When
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.http import Http404
from django.urls import reverse
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _
from hashids import Hashids
from pyotp import TOTP

from .exceptions import PermissionError
from ..audit.auditlog import log
from ..audit.models import AuditLogCategoryChoices, LogEntry


class AccessPermissionTypes(models.IntegerChoices):
    NOT_ALLOWED = 0
    ALLOWED = 1
    TEMPORARILY_ALLOWED = 2
    SUPERUSER_ALLOWED = 3


def validate_url(value):
    if "://" not in value or \
            value.startswith("javascript:") or \
            value.startswith("data:"):
        raise ValidationError(_("invalid URL"))


class HashIDModel(models.Model):
    hashid = models.CharField(
        max_length=24,
        null=True,
        unique=True,
    )

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        if not self.pk:
            # need to save once to get a primary key, then once again to
            # save the hashid
            super(HashIDModel, self).save(*args, **kwargs)
        if not self.hashid:
            hasher = Hashids(
                min_length=settings.HASHID_MIN_LENGTH,
                salt=self.HASHID_NAMESPACE + settings.HASHID_SALT,
            )
            self.hashid = hasher.encode(self.pk)
        # we cannot force insert anymore because we might already have
        # created the object
        kwargs['force_insert'] = False
        return super(HashIDModel, self).save(*args, **kwargs)


class Secret(HashIDModel):
    HASHID_NAMESPACE = "Secret"

    ACCESS_POLICY_DISCOVERABLE = 1
    ACCESS_POLICY_ANY = 2
    ACCESS_POLICY_HIDDEN = 3
    ACCESS_POLICY_CHOICES = (
        (ACCESS_POLICY_DISCOVERABLE, _("discoverable")),
        (ACCESS_POLICY_ANY, _("everyone")),
        (ACCESS_POLICY_HIDDEN, _("hidden")),
    )
    CONTENT_PASSWORD = 1
    CONTENT_CC = 2
    CONTENT_FILE = 3
    CONTENT_CHOICES = (
        (CONTENT_PASSWORD, _("Password")),
        (CONTENT_CC, _("Credit Card")),
        (CONTENT_FILE, _("File")),
    )
    STATUS_OK = 1
    STATUS_NEEDS_CHANGING = 2
    STATUS_DELETED = 3
    STATUS_CHOICES = (
        (STATUS_OK, _("OK")),
        (STATUS_NEEDS_CHANGING, _("needs changing")),
        (STATUS_DELETED, _("deleted")),
    )

    access_policy = models.PositiveSmallIntegerField(
        choices=ACCESS_POLICY_CHOICES,
        default=ACCESS_POLICY_DISCOVERABLE,
    )
    content_type = models.PositiveSmallIntegerField(
        choices=CONTENT_CHOICES,
        default=CONTENT_PASSWORD,
    )
    created = models.DateTimeField(auto_now_add=True)
    created_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        models.PROTECT,
        related_name='passwords_created',
    )
    current_revision = models.ForeignKey(
        'SecretRevision',
        models.PROTECT,
        blank=True,
        null=True,
        related_name='_password_current_revision',
    )
    description = models.TextField(
        blank=True,
        null=True,
        help_text=_('Further information on the secret.'),
    )
    filename = models.CharField(
        blank=True,
        max_length=255,
        null=True,
    )
    last_changed = models.DateTimeField(
        auto_now_add=True,
    )
    last_read = models.DateTimeField(
        default=now,
    )
    name = models.CharField(max_length=92, help_text=_('Enter a unique name for the secret'))
    needs_changing_on_leave = models.BooleanField(
        default=True,
    )
    shared_groups = models.ManyToManyField(
        Group,
        blank=True,
        through='SharedSecretData',
        through_fields=('secret', 'group'),
    )
    shared_users = models.ManyToManyField(
        settings.AUTH_USER_MODEL,
        blank=True,
        through='SharedSecretData',
        through_fields=('secret', 'user'),
    )
    status = models.PositiveSmallIntegerField(
        choices=STATUS_CHOICES,
        default=STATUS_OK,
    )
    url = models.CharField(
        blank=True,
        max_length=255,
        null=True,
        # Django's builtin URL validation is pretty strict to the point
        # of rejecting perfectly good URLs, thus we roll our own very
        # liberal validation
        validators=[validate_url],
    )
    username = models.CharField(
        blank=True,
        max_length=255,
        null=True,
    )

    search_index = SearchVectorField(blank=True, null=True)

    class Meta:
        ordering = ('name', 'username')

    def __str__(self):
        return self.name

    def __repr__(self):
        return "<Secret '{name}' ({id})>".format(id=self.hashid, name=self.name)

    def check_read_access(self, user):
        if not self.is_visible_to_user(user):
            raise Http404

        readable = self.is_readable_by_user(user)
        if not readable:
            raise PermissionDenied()
        return readable

    def check_share_access(self, user):
        if not self.is_visible_to_user(user):
            raise Http404

        shareable = self.is_shareable_by_user(user)
        if not shareable:
            raise PermissionDenied()
        return shareable

    @property
    def full_url(self):
        return settings.BASE_URL.rstrip("/") + self.get_absolute_url()

    def get_absolute_url(self):
        return reverse('secrets.secret-detail', args=[str(self.hashid)])

    def get_data(self, user):
        if not self.current_revision:
            raise Http404

        read_allowed = self.is_readable_by_user(user)
        if not read_allowed:
            log(
                _("{user} tried to access '{name}' without permission").format(name=self.name, user=user.username),
                actor=user,
                category=AuditLogCategoryChoices.SECRET_PERMISSION_VIOLATION,
                level='warning',
                secret=self,
            )
            raise PermissionError(_(
                "{user} not allowed access to '{name}' ({id})"
            ).format(
                id=self.id,
                name=self.name,
                user=user.username,
            ))
        if read_allowed == AccessPermissionTypes.SUPERUSER_ALLOWED:
            category = AuditLogCategoryChoices.SECRET_ELEVATED_SUPERUSER_READ
            log_message = _("{user} used superuser privileges to read '{name}'")
        else:
            category = AuditLogCategoryChoices.SECRET_READ
            log_message = _("{user} read '{name}'")
        log(
            log_message.format(
                name=self.name,
                user=user.username,
            ),
            actor=user,
            category=category,
            level='info',
            secret=self,
            secret_revision=self.current_revision,
        )
        self.current_revision.accessed_by.add(user)
        self.current_revision.save()
        self.last_read = now()
        self.save()
        f = Fernet(settings.TEAMVAULT_SECRET_KEY)
        plaintext_data = self.current_revision.encrypted_data
        plaintext_data = f.decrypt(plaintext_data).decode('utf-8')
        try:
            plaintext_data = loads(plaintext_data)
            if self.content_type == Secret.CONTENT_FILE:
                plaintext_data = plaintext_data["file_content"]
        except JSONDecodeError:
            if self.content_type == self.CONTENT_PASSWORD:
                plaintext_data = dict(password=plaintext_data)

        return plaintext_data

    def get_otp(self, request):
        if request.session.get("otp_key_data"):
            data = request.session["otp_key_data"]
        else:
            data = self.get_data(request.user)
        otp_key = data['otp_key']
        digits = int(data['digits'])
        request.session["otp_key_data"] = {'otp_key': otp_key, 'digits': digits}
        totp = TOTP(otp_key, digits=digits)
        return totp.now()

    @classmethod
    def get_all_readable_by_user(cls, user):
        if user.is_superuser and settings.ALLOW_SUPERUSER_READS:
            return cls.objects.all()

        allowed_shares = SharedSecretData.objects.with_expiry_state().filter(
            Q(user=user) | Q(group__user=user)
        ).exclude(is_expired=True).values('secret__pk')
        return cls.objects.filter(
            Q(access_policy=cls.ACCESS_POLICY_ANY) | Q(pk__in=allowed_shares)
        ).exclude(status=cls.STATUS_DELETED).distinct()

    @classmethod
    def get_all_visible_to_user(cls, user, queryset=None):
        if queryset is None:
            queryset = cls.objects.all()

        if user.is_superuser:
            return queryset

        allowed_shares = SharedSecretData.objects.with_expiry_state().filter(
            Q(user=user) | Q(group__user=user)
        ).exclude(is_expired=True).values('secret__pk')
        return queryset.filter(
            Q(access_policy__in=(cls.ACCESS_POLICY_ANY, cls.ACCESS_POLICY_DISCOVERABLE)) | Q(pk__in=allowed_shares)
        ).exclude(status=cls.STATUS_DELETED).distinct()

    @classmethod
    def get_most_used_for_user(cls, user, limit=5):
        since = now() - timedelta(days=90)
        accessed_secrets = LogEntry.objects.filter(
            actor=user,
            secret__isnull=False,
            time__gte=since,
        ).order_by(
            'secret'
        ).values(
            'secret'
        ).annotate(
            access_count=models.Count('secret'),
        )
        ordered_secrets = sorted(accessed_secrets, key=itemgetter('access_count'), reverse=True)
        return [cls.objects.get(id=item['secret']) for item in ordered_secrets[:limit]]

    @classmethod
    def get_most_recently_used_for_user(cls, user, limit=5):
        log_entries = LogEntry.objects.filter(
            actor=user
        ).values(
            'secret'
        ).annotate(
            latest_time=Max('time')
        ).order_by(
            '-latest_time'
        )[:limit]

        ordered_secret_ids = [access['secret'] for access in log_entries]
        unordered_secrets = Secret.objects.filter(id__in=ordered_secret_ids)
        secret_map = {secret.id: secret for secret in unordered_secrets}
        ordered_secrets = [secret_map[secret_id] for secret_id in ordered_secret_ids if secret_id in secret_map]
        return ordered_secrets

    @classmethod
    def get_search_results(cls, user, term, limit=None, substr_search=True):
        base_queryset = cls.get_all_visible_to_user(user)
        name_hits = base_queryset.filter(name__icontains=term)
        fulltext_hits = cls.get_all_visible_to_user(
            user,
            queryset=cls.objects.filter(search_index=term),
        )

        substr_hits = Secret.objects.none()
        if substr_search:
            substr_hits = base_queryset.filter(
                models.Q(filename__icontains=term) |
                models.Q(url__icontains=term) |
                models.Q(username__icontains=term) |
                models.Q(hashid__exact=term)
            )
        if limit:
            name_hits = name_hits[:limit]
            fulltext_hits = fulltext_hits[:limit]
            substr_hits = substr_hits[:limit]
        # concatenate and remove duplicates
        result = (name_hits | fulltext_hits | substr_hits).distinct()
        if limit:
            return result[:limit]
        else:
            return result

    def is_readable_by_user(self, user):
        if self.status != self.STATUS_DELETED:
            shares = self.share_data.with_expiry_state().filter(
                Q(user=user) | Q(group__user=user)
            ).exclude(is_expired=True)

            if self.access_policy == self.ACCESS_POLICY_ANY or shares.filter(granted_until__isnull=True).exists():
                return AccessPermissionTypes.ALLOWED

            if shares.exists():
                return AccessPermissionTypes.TEMPORARILY_ALLOWED

        if user.is_superuser and settings.ALLOW_SUPERUSER_READS:
            return AccessPermissionTypes.SUPERUSER_ALLOWED

        return AccessPermissionTypes.NOT_ALLOWED

    def is_shareable_by_user(self, user):
        read_permission = self.is_readable_by_user(user)
        if read_permission == AccessPermissionTypes.ALLOWED:
            return AccessPermissionTypes.ALLOWED

        if user.is_superuser:
            return AccessPermissionTypes.SUPERUSER_ALLOWED
        return AccessPermissionTypes.NOT_ALLOWED

    def is_visible_to_user(self, user):
        if self.status != self.STATUS_DELETED:
            if (
                    self.access_policy in (self.ACCESS_POLICY_ANY, self.ACCESS_POLICY_DISCOVERABLE) or
                    self.is_readable_by_user(user)
            ):
                return AccessPermissionTypes.ALLOWED

        if user.is_superuser:
            return AccessPermissionTypes.SUPERUSER_ALLOWED
        return AccessPermissionTypes.NOT_ALLOWED

    def set_data(self, user, plaintext_data, skip_access_check=False):
        # skip_access_check is used when initially creating a secret
        # and makes it possible to create secrets you don't have access
        # to
        if not skip_access_check and not self.is_readable_by_user(user):
            raise PermissionError(_(
                "{user} not allowed access to '{name}' ({id})"
            ).format(
                id=self.id,
                name=self.name,
                user=user.username,
            ))
        # save the length before encoding so multi-byte characters don't
        # mess up the result
        set_password = "password" in plaintext_data and self.content_type == Secret.CONTENT_PASSWORD
        set_otp = "otp_key" in plaintext_data
        plaintext_length = len(plaintext_data)
        f = Fernet(settings.TEAMVAULT_SECRET_KEY)
        if set_password:
            plaintext_data_sha256 = sha256(plaintext_data["password"].encode('utf-8')).hexdigest()
        else:
            plaintext_data_sha256 = sha256(dumps(plaintext_data).encode('utf-8')).hexdigest()
        try:
            # see the comment on unique_together for SecretRevision
            p = SecretRevision.objects.get(
                plaintext_data_sha256=plaintext_data_sha256,
                secret=self,
            )
        except SecretRevision.DoesNotExist:
            p = SecretRevision()
        if self.current_revision and self.content_type == Secret.CONTENT_PASSWORD:
            old_data = f.decrypt(self.current_revision.encrypted_data).decode('utf-8')
            # If not already dict convert to dict since password and otp key are now stored together in dict format.
            # To keep everything uniform CC and file secrets stored as a dict as well
            try:
                old_data = loads(old_data)
            except JSONDecodeError:
                old_data = dict(password=old_data)
            if not set_password:
                plaintext_data["password"] = old_data["password"]
            if not set_otp and "otp_key" in old_data:
                for key in ["otp_key", "digits", "algorithm"]:
                    plaintext_data[key] = old_data[key]
                    set_otp = True
        if self.content_type == Secret.CONTENT_PASSWORD and "password" in plaintext_data.keys():
            plaintext_length = len(plaintext_data["password"])
        if set_otp:
            p.otp_key_set = True
        plaintext_data = dumps(plaintext_data).encode('utf-8')
        p.encrypted_data = f.encrypt(plaintext_data)
        p.length = plaintext_length
        p.plaintext_data_sha256 = plaintext_data_sha256
        p.set_by = user
        p.secret = self
        p.save()
        p.accessed_by.add(user)

        if self.current_revision:
            previous_revision_id = self.current_revision.id
        else:
            previous_revision_id = _("none")
        self.current_revision = p
        self.last_changed = now()
        self.last_read = now()
        if self.status == self.STATUS_NEEDS_CHANGING:
            self.status = self.STATUS_OK
        self.save()
        log(
            _("{user} set a new secret for '{name}' ({oldrev}->{newrev})").format(
                name=self.name,
                newrev=self.current_revision.id,
                oldrev=previous_revision_id,
                user=user.username,
            ),
            actor=user,
            category=AuditLogCategoryChoices.SECRET_CHANGED,
            level='info',
            secret=self,
            secret_revision=self.current_revision,
        )

    def needs_changing(self):
        return self.status == self.STATUS_NEEDS_CHANGING

    def share(self, grant_description, granted_by, user=None, group=None, granted_until=None):
        if (user and group) or (not user and not group):
            raise ValueError('Specify either a user or a group!')

        if not isinstance(granted_by, User):
            raise ValueError('granted_by has to be a User object!')

        permission = self.is_shareable_by_user(granted_by)
        if not permission:
            raise PermissionDenied()

        share_obj = self.share_data.create(
            grant_description=grant_description,
            granted_by=granted_by,
            granted_until=granted_until,
            group=group,
            user=user,
        )
        log(
            _("{user} granted access to {shared_entity_type} '{name}' {time}").format(
                shared_entity_type=share_obj.shared_entity_type,
                name=share_obj.shared_entity_name,
                user=granted_by.username,
                time=_('until ') + share_obj.granted_until.isoformat() if share_obj.granted_until else _('permanently')
            ),
            actor=granted_by,
            category=(
                AuditLogCategoryChoices.SECRET_SUPERUSER_SHARED
                if permission == AccessPermissionTypes.SUPERUSER_ALLOWED
                else AuditLogCategoryChoices.SECRET_SHARED
            ),
            level='warning',
            reason=grant_description,
            secret=self,
        )

        return share_obj


class SecretRevision(HashIDModel):
    HASHID_NAMESPACE = "SecretRevision"

    accessed_by = models.ManyToManyField(
        settings.AUTH_USER_MODEL,
    )
    created = models.DateTimeField(auto_now_add=True)
    encrypted_data = models.BinaryField()
    otp_key_set = models.BooleanField(default=False)
    length = models.PositiveIntegerField(
        default=0,
    )
    plaintext_data_sha256 = models.CharField(
        max_length=64,
    )
    secret = models.ForeignKey(
        Secret,
        models.PROTECT,
    )
    set_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        models.PROTECT,
        related_name='password_revisions_set',
    )

    class Meta:
        ordering = ('-created',)
        # Consider the following scenario:
        # 1. an employee reads a password, it is "secret123"
        # 2. the password is changed to "secret234"
        # 3. the password is changed back to "secret123"
        # even though the employee never read the third instance of the
        # password, they still know the password from step 1.
        # This unique_together forces passwords 1 and 3 to be the same
        # database object, retaining accessed_by. This way, when the
        # employee leaves, password 3 is correctly assumed known to
        # the employee.
        unique_together = (('plaintext_data_sha256', 'secret'),)

    def __repr__(self):
        return "<SecretRevision '{name}' ({id})>".format(id=self.hashid, name=self.secret.name)

    @property
    def is_current_revision(self):
        return self.secret.current_revision == self


class SecretShareQuerySet(models.QuerySet):
    # TODO: Rename to group_shares, user_shares
    def groups(self):
        return self.with_expiry_state().filter(group__isnull=False).order_by('group__name')

    def users(self):
        return self.with_expiry_state().filter(user__isnull=False).order_by('user__username')

    def with_expiry_state(self):
        return self.annotate(
            is_expired=Case(
                When(
                    granted_until__lte=now(),
                    then=Value(True)
                ),
                default=Value(False),
                output_field=BooleanField()
            )
        )


class SharedSecretData(models.Model):
    objects = SecretShareQuerySet.as_manager()

    group = models.ForeignKey(
        Group,
        on_delete=models.CASCADE,
        null=True,
        related_name='secret_share_data',
    )

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        null=True,
        related_name='secret_share_data',
    )

    secret = models.ForeignKey(
        'Secret',
        on_delete=models.CASCADE,
        related_name='share_data',
    )

    grant_description = models.TextField(
        null=True
    )

    granted_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.PROTECT,
        null=True,
        related_name='+',
    )

    granted_on = models.DateTimeField(
        auto_now_add=True,
        null=True,
    )

    # Secrets are considered permanently shared if granted_until is not set
    granted_until = models.DateTimeField(
        blank=True,
        null=True,
    )

    @property
    def shared_entity(self):
        return self.group if self.group else self.user

    @property
    def shared_entity_name(self):
        return self.group.name if self.group else self.user.username

    @property
    def shared_entity_type(self):
        return 'group' if self.group else 'user'

    @property
    def about_to_expire(self):
        if not self.granted_until:
            return False

        if now() + timedelta(hours=8) >= self.granted_until:
            return True

    @property
    def expiry_icon(self):
        if not self.granted_until:
            return 'success'

        if self.about_to_expire:
            time_left = self.granted_until - now()
            if time_left <= timedelta(hours=8):
                return 'danger'
        return 'warning'

    def __str__(self):
        return f'SharedSecretData object ({self.secret.name}: {self.user.username if self.user else self.group.name})'

    class Meta:
        constraints = [
            models.CheckConstraint(
                check=(
                        (Q(group__isnull=False) & Q(user__isnull=True)) |
                        (Q(group__isnull=True) & Q(user__isnull=False))
                ),
                name='only_one_set'
            ),
        ]
        unique_together = [('group', 'secret'), ('user', 'secret')]


@receiver(post_save, sender=Secret)
def update_search_index(sender, **kwargs):
    Secret.objects.filter(id=kwargs['instance'].id).update(
        search_index=(
                SearchVector('name', weight='A') +
                SearchVector('description', weight='B') +
                SearchVector('username', weight='C') +
                SearchVector('filename', weight='D')
        ),
    )



# Generated by Django 2.2.4 on 2019-08-22 12:34

from django.db import migrations


def backfill_last_changed(apps, schema_editor):
    # We can't import the Person model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    Secret = apps.get_model('secrets', 'Secret')
    for secret in Secret.objects.all():
        if secret.current_revision:  # leftovers from a bug
            secret.last_changed = secret.current_revision.created
            secret.save()


class Migration(migrations.Migration):

    dependencies = [
        ('secrets', '0022_secret_last_changed'),
    ]

    operations = [
        migrations.RunPython(backfill_last_changed),
    ]



# Generated by Django 2.0 on 2018-02-20 13:56

from django.db import migrations


def copy_owner_data(apps, schema_editor):
    Secret = apps.get_model('secrets', 'Secret')
    for secret in Secret.objects.all():
        secret.allowed_groups.add(*list(secret.owner_groups.all()))
        secret.allowed_users.add(*list(secret.owner_users.all()))
        secret.notify_on_access_request.add(*list(secret.owner_users.all()))
        for group in secret.owner_groups.all():
            secret.notify_on_access_request.add(*list(group.user_set.all()))


class Migration(migrations.Migration):

    dependencies = [
        ('secrets', '0019_secret_notify_on_access_request'),
    ]

    operations = [
        migrations.RunPython(copy_owner_data),
    ]



#!/usr/bin/env python3
import os
import sys
from os.path import dirname, join, realpath

if __name__ == "__main__":
    sys.path.append(join(realpath(dirname(dirname(__file__)))))
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "teamvault.settings")

    from django.core.management import execute_from_command_line

    execute_from_command_line(sys.argv)



# Generated by Django 4.2.3 on 2023-07-11 18:56

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("accounts", "0002_add_user_settings"),
    ]

    operations = [
        migrations.AddField(
            model_name="usersettings",
            name="avatar",
            field=models.BinaryField(blank=True, null=True),
        ),
    ]



# Generated by Django 4.2.3 on 2024-03-05 14:15

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("audit", "0006_logentry_reason"),
    ]

    operations = [
        migrations.AlterField(
            model_name="logentry",
            name="category",
            field=models.CharField(
                choices=[
                    ("secret_read", "secret_read"),
                    ("secret_elevated_superuser_read", "secret_elevated_superuser_read"),
                    ("secret_permission_violation", "secret_permission_violation"),
                    ("secret_changed", "secret_changed"),
                    ("secret_needs_changing_reminder", "secret_needs_changing_reminder"),
                    ("secret_shared", "secret_shared"),
                    ("secret_superuser_shared", "secret_superuser_shared"),
                    ("secret_share_removed", "secret_share_removed"),
                    ("secret_superuser_share_removed", "secret_superuser_share_removed"),
                    ("secret_legacy_access_requests", "secret_legacy_access_requests"),
                    ("user_activated", "user_activated"),
                    ("user_deactivated", "user_deactivated"),
                    ("user_settings_changed", "user_settings_changed"),
                    ("share_automatically_revoked", "share_automatically_revoked"),
                    ("miscellaneous", "miscellaneous"),
                ],
                default="miscellaneous",
                max_length=64,
            ),
        ),
    ]



# Generated by Django 4.2.3 on 2023-07-07 13:22

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("auth", "0012_alter_user_first_name_max_length"),
        ("secrets", "0030_sharedsecretdata_granted_on"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="secret",
            name="allowed_groups",
        ),
        migrations.RemoveField(
            model_name="secret",
            name="allowed_users",
        ),
        migrations.AddField(
            model_name="secret",
            name="shared_groups",
            field=models.ManyToManyField(
                blank=True, through="secrets.SharedSecretData", to="auth.group"
            ),
        ),
        migrations.AddField(
            model_name="secret",
            name="shared_users",
            field=models.ManyToManyField(
                blank=True,
                through="secrets.SharedSecretData",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="sharedsecretdata",
            name="granted_by",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="sharedsecretdata",
            name="group",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="secret_share_data",
                to="auth.group",
            ),
        ),
        migrations.AlterField(
            model_name="sharedsecretdata",
            name="secret",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="share_data",
                to="secrets.secret",
            ),
        ),
        migrations.AlterField(
            model_name="sharedsecretdata",
            name="user",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="secret_share_data",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]



from base64 import b64encode
from json import loads

from django.conf import settings
from django.shortcuts import get_object_or_404
from django.utils.translation import gettext_lazy as _
from rest_framework import generics
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.exceptions import PermissionDenied
from rest_framework.response import Response

from teamvault.apps.audit.auditlog import log
from teamvault.apps.audit.models import AuditLogCategoryChoices
from .serializers import SecretRevisionSerializer, SecretSerializer, SharedSecretDataSerializer
from ..models import Secret, SecretRevision, SharedSecretData, AccessPermissionTypes
from ..utils import generate_password


class SecretDetail(generics.RetrieveUpdateDestroyAPIView):
    model = Secret
    serializer_class = SecretSerializer

    def destroy(self, request, *args, **kwargs):
        obj = self.get_object()
        obj.status = Secret.STATUS_DELETED
        obj.save()
        return Response(status=status.HTTP_204_NO_CONTENT)

    def get_object(self):
        obj = get_object_or_404(Secret, hashid=self.kwargs['hashid'])
        if not obj.is_visible_to_user(self.request.user):
            self.permission_denied(self.request)
        return obj

    def perform_update(self, serializer):
        instance = serializer.save()
        if hasattr(instance, '_data'):
            instance.set_data(self.request.user, instance._data)
            del instance._data


class SecretList(generics.ListCreateAPIView):
    model = Secret
    serializer_class = SecretSerializer

    def get_queryset(self):
        if 'search' in self.request.query_params:
            return Secret.get_search_results(
                self.request.user,
                self.request.query_params['search'],
            )
        else:
            return Secret.get_all_visible_to_user(self.request.user)

    def perform_create(self, serializer):
        instance = serializer.save(created_by=self.request.user)
        if hasattr(instance, '_data'):
            instance.set_data(self.request.user, instance._data, skip_access_check=True)
            del instance._data


class SecretRevisionDetail(generics.RetrieveAPIView):
    model = SecretRevision
    serializer_class = SecretRevisionSerializer

    def get_object(self):
        obj = get_object_or_404(SecretRevision, hashid=self.kwargs['hashid'])
        obj.secret.check_read_access(self.request.user)
        return obj


class SecretShare(generics.ListCreateAPIView):
    model = SharedSecretData
    serializer_class = SharedSecretDataSerializer

    def get_object(self):
        obj = get_object_or_404(Secret, hashid=self.kwargs['hashid'])
        return obj

    def get_queryset(self):
        obj = self.get_object()
        return obj.share_data.all()

    def get_serializer_context(self):
        context = super().get_serializer_context()
        context['secret'] = self.get_object()
        return context

    def perform_create(self, serializer):
        secret = self.get_object()
        permission = secret.is_shareable_by_user(self.request.user)
        if not permission:
            raise PermissionDenied

        obj = serializer.save(granted_by=self.request.user, secret=secret)
        log(
            _("{user} granted access to {shared_entity_type} '{name}' {time}").format(
                shared_entity_type=obj.shared_entity_type,
                name=obj.shared_entity_name,
                user=self.request.user.username,
                time=_('until ') + obj.granted_until.isoformat() if obj.granted_until else _('permanently')
            ),
            actor=self.request.user,
            category=(
                AuditLogCategoryChoices.SECRET_SUPERUSER_SHARED
                if permission == AccessPermissionTypes.SUPERUSER_ALLOWED
                else AuditLogCategoryChoices.SECRET_SHARED
            ),
            level='warning',
            secret=secret,
        )


class SecretShareDetail(generics.RetrieveDestroyAPIView):
    model = SharedSecretData
    serializer_class = SharedSecretDataSerializer

    def get_object(self):
        obj = get_object_or_404(SharedSecretData, secret__hashid=self.kwargs['hashid'], pk=self.kwargs['pk'])
        return obj

    def perform_destroy(self, instance):
        permission = instance.secret.is_shareable_by_user(self.request.user)
        if not permission:
            raise PermissionDenied()

        secret = instance.secret
        entity_type = instance.shared_entity_type
        entity_name = instance.shared_entity_name
        instance.delete()

        log(
            _("{user} removed access of {shared_entity_type} '{name}'").format(
                shared_entity_type=entity_type,
                name=entity_name,
                user=self.request.user.username,
            ),
            actor=self.request.user,
            category=(
                AuditLogCategoryChoices.SECRET_SUPERUSER_SHARE_REMOVED
                if permission == AccessPermissionTypes.SUPERUSER_ALLOWED
                else AuditLogCategoryChoices.SECRET_SHARE_REMOVED
            ),
            level='warning',
            secret=secret,
        )


@api_view(['GET'])
def data_get(request, hashid):
    secret_revision = get_object_or_404(SecretRevision, hashid=hashid)
    secret_revision.secret.check_read_access(request.user)
    data = secret_revision.secret.get_data(request.user)
    if secret_revision.secret.content_type == Secret.CONTENT_PASSWORD:
        return Response({'password': data["password"]})
    elif secret_revision.secret.content_type == Secret.CONTENT_FILE:
        return Response({'file': b64encode(data).decode('ascii')})
    elif secret_revision.secret.content_type == Secret.CONTENT_CC:
        return Response(data)


@api_view(['GET'])
def generate_password_view(*_args, **_kwargs):
    return Response(generate_password(
        settings.PASSWORD_LENGTH,
        settings.PASSWORD_DIGITS,
        settings.PASSWORD_UPPER,
        settings.PASSWORD_LOWER,
        settings.PASSWORD_SPECIAL
    ))


@api_view(['GET'])
def otp_get(request, hashid):
    secret_revision = get_object_or_404(SecretRevision, hashid=hashid)
    secret = secret_revision.secret
    secret.check_read_access(request.user)
    otp = secret.get_otp(request)
    return Response(otp)



from django.apps import AppConfig


class SettingsConfig(AppConfig):
    name = 'teamvault.apps.settings'

    def ready(self):
        from django.conf import settings
        from . import config, webpack
        parsed_config = config.get_config()
        config.configure_base_url(parsed_config, settings)
        config.configure_debugging(parsed_config, settings)
        config.configure_ldap_auth(parsed_config, settings)
        config.configure_google_auth(parsed_config, settings)
        config.configure_max_file_size(parsed_config, settings)
        config.configure_password_generator(parsed_config, settings)
        config.configure_superuser_reads(parsed_config, settings)
        config.configure_teamvault_secret_key(parsed_config, settings)
        config.configure_password_update_alert(parsed_config, settings)
        config.configure_whitenoise(settings)
        webpack.configure_webpack(settings)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('secrets', '0005_secret_search_index'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='secret',
            options={'ordering': ('name', 'username')},
        ),
    ]



from django.contrib.auth.decorators import user_passes_test
from django.db.models import Q
from django.views.generic import ListView

from .filters import AuditLogFilter
from .models import LogEntry
from ..secrets.models import Secret
from ...views import FilterMixin


class LogEntryList(ListView, FilterMixin):
    filter = None
    filter_class = AuditLogFilter
    context_object_name = 'log_entries'
    paginate_by = 25
    template_name = "audit/log.html"

    def get_queryset(self):
        queryset = LogEntry.objects.all()
        if "search" in self.request.GET:
            query = self.request.GET['search']
            queryset = queryset.filter(
                Q(actor__icontains=query) |
                Q(message__icontains=query)
            )

        return self.get_filtered_queryset(queryset)

    @staticmethod
    def manipulate_filter_form(bound_data, filter_form):
        # Set queryset since we'll retrieve choices via ajax and need to show the initial one
        if bound_data.get('secret'):
            secret_choices = Secret.objects.filter(pk=bound_data['secret'].pk)
        else:
            secret_choices = Secret.objects.none()
        filter_form.fields['secret'].queryset = secret_choices
        return filter_form


auditlog = user_passes_test(lambda u: u.is_superuser)(LogEntryList.as_view())



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.postgres.operations import UnaccentExtension
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('secrets', '0003_auto_20150113_1915'),
    ]
    operations = [
        UnaccentExtension(),
    ]



# Generated by Django 4.2.1 on 2023-06-11 22:56

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("secrets", "0027_sharedsecretdata_only_one_set"),
    ]

    operations = [
        migrations.AddField(
            model_name="sharedsecretdata",
            name="grant_description",
            field=models.TextField(null=True),
        ),
        migrations.AddField(
            model_name="sharedsecretdata",
            name="granted_until",
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



import logging
from base64 import b64encode
from hashlib import md5

import requests

from teamvault.apps.accounts.models import UserProfile as UserProfileModel, UserProfile
from teamvault.apps.audit.models import LogEntry
from teamvault.apps.secrets.models import SharedSecretData, Secret, SecretRevision

logger = logging.getLogger(__name__)


def save_gravatar(user, *_args, **_kwargs):
    email_hash = md5(user.email.strip().lower().encode("utf-8")).hexdigest()
    resp = requests.get(f'https://gravatar.com/avatar/{email_hash}?s=200&r=g&d=mp')
    if resp.ok:
        user_settings = UserProfileModel.objects.get_or_create(user=user)[0]
        user_settings.avatar = b64encode(resp.content)
        user_settings.save()


def save_google_avatar(response, user, *_args, **_kwargs):
    resp = requests.get(response['picture'])
    if resp.ok:
        user_settings = UserProfileModel.objects.get_or_create(user=user)[0]
        user_settings.avatar = b64encode(resp.content)
        user_settings.save()


def merge_users(user1, user2, dry_run=True):
    logger.info(
        f'Merging user {user1.username} into {user2.username}\n'
        f'Secrets & Audit Logs will be merged. User Profiles, Social Auth data and User itself will be deleted.\n'
        f'Dry run: {dry_run}'
    )

    # Secrets / SharedSecretData / SecretRevisions
    user1_secrets = SharedSecretData.objects.filter(user=user1)
    user2_secrets = SharedSecretData.objects.filter(user=user2)
    secrets_to_merge = user1_secrets.exclude(pk__in=user2_secrets.values_list('pk', flat=True))
    logger.info(f'{secrets_to_merge.count()} Secrets found: {secrets_to_merge.values_list("pk", flat=True)}')

    user1_created = Secret.objects.filter(created_by=user1)
    logger.info(f'{user1_created.count()} Secrets w/ created_by found: {user1_created.values_list("pk", flat=True)}')

    user1_revisions = SecretRevision.objects.filter(set_by=user1)
    logger.info(f'{user1_revisions.count()} SecretRevisions found: {user1_revisions.values_list("pk", flat=True)}')

    # Audit Logs
    user1_actor_logs = LogEntry.objects.filter(actor=user1)
    user1_user_logs = LogEntry.objects.filter(user=user1)
    logger.info(f'{user1_actor_logs.count()} Actor Logs found: {user1_actor_logs.values_list("pk", flat=True)}')
    logger.info(f'{user1_user_logs.count()} User Logs found: {user1_user_logs.values_list("pk", flat=True)}')

    # User Profiles
    user1_profiles = UserProfile.objects.filter(user=user1)
    logger.info(f'{user1_profiles.count()} User Profiles found: {user1_profiles.values_list("pk", flat=True)}')

    # User Social Auth data
    user1_social_data = user1.social_auth.all().exclude(pk__in=user2.social_auth.all().values_list('pk', flat=True))
    logger.info(f'{user1_social_data.count()} Social Auth data found: {user1_social_data.values_list("pk", flat=True)}')

    if not dry_run:
        secrets_to_merge.update(user=user2)
        user1_created.update(created_by=user2)
        user1_revisions.update(set_by=user2)
        logger.info('Updated secrets.')

        user1_actor_logs.update(actor=user2)
        user1_user_logs.update(user=user2)
        logger.info('Updated logs.')

        user1_profiles.delete()
        logger.info('Deleted User Profiles.')

        user1_social_data.delete()
        logger.info('Deleted Social Auth data.')

        user1.delete()
        logger.info('Deleted User')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import teamvault.apps.secrets.models


class Migration(migrations.Migration):

    dependencies = [
        ('secrets', '0006_auto_20150124_1103'),
    ]

    operations = [
        migrations.AlterField(
            model_name='secret',
            name='url',
            field=models.CharField(blank=True, null=True, validators=[teamvault.apps.secrets.models.validate_url], max_length=255),
        ),
    ]



from django import forms

from teamvault.apps.accounts.models import UserProfile


class UserProfileForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['default_sharing_groups'].queryset = self.fields['default_sharing_groups'].queryset.order_by('name')

    class Meta:
        fields = ['default_sharing_groups', 'hide_deleted_secrets']
        model = UserProfile



from django.contrib import messages
from django.contrib.messages.storage.base import BaseStorage, Message
from django_htmx.http import trigger_client_event


def htmx_message_middleware(get_response):
    # One-time configuration and initialization.

    def middleware(request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.
        response = get_response(request)

        # Ignore non-HTMX requests
        if "HX-Request" not in request.headers:
            return response

        # HTMX will not read HX headers in redirects but the subsequent GET response.
        if 300 <= response.status_code < 400:
            return response

        storage: BaseStorage = messages.get_messages(request)
        msg_list = []
        for msg in storage:
            msg: Message
            msg_list.append({
                'message': msg.message,
                # debug|info|success|warning|error
                'level': msg.level_tag,
            })

        trigger_client_event(response, 'django.contrib.messages', {'message_list': msg_list})
        return response

    return middleware



import django_filters
from django import forms
from django.contrib.auth.models import User
from django.utils.translation import gettext_lazy as _

from .models import LogEntry, AuditLogCategoryChoices
from ..secrets.models import Secret


class AuditLogFilter(django_filters.FilterSet):
    actor = django_filters.ModelChoiceFilter(
        to_field_name='username',
        queryset=User.objects.all().order_by('username'),
    )
    category = django_filters.MultipleChoiceFilter(
        choices=AuditLogCategoryChoices.choices,
        label=_('Categories'),
        widget=forms.CheckboxSelectMultiple(),
    )
    secret = django_filters.ModelChoiceFilter(
        field_name='secret',
        to_field_name='hashid',
        queryset=Secret.objects.all(),
    )
    user = django_filters.ModelChoiceFilter(
        to_field_name='username',
        queryset=User.objects.all().order_by('username'),
    )

    class Meta:
        model = LogEntry
        fields = ['secret', 'actor', 'user']



# Generated by Django 4.2.1 on 2023-06-11 21:27

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


def copy_shared_secrets_data(apps, schema_editor):
    secret_model = apps.get_model('secrets', 'Secret')
    shared_secret_data_model = apps.get_model('secrets', 'SharedSecretData')

    secret_data = secret_model.objects.all().only(
        'id',
        'allowed_users',
        'allowed_groups'
    )

    user_shares = []
    group_shares = []
    for secret in secret_data:
        group_shares += [(secret, group) for group in secret.allowed_groups.all()]
        user_shares += [(secret, user) for user in secret.allowed_users.all()]

    shared_secret_data_model.objects.bulk_create(
        [shared_secret_data_model(secret=secret, user=user) for secret, user in user_shares] +
        [shared_secret_data_model(secret=secret, group=group) for secret, group in group_shares]
    )


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('auth', '0012_alter_user_first_name_max_length'),
        ('secrets', '0025_alter_secret_description_alter_secret_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='SharedSecretData',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('group', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='auth.group')),
                ('secret', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='secrets.secret')),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'unique_together': {('user', 'secret'), ('group', 'secret')},
            },
        ),
        migrations.RunPython(
            code=copy_shared_secrets_data,
        ),
        migrations.RemoveField(
            model_name='secret',
            name='allowed_groups',
        ),
        migrations.RemoveField(
            model_name='secret',
            name='allowed_users',
        ),
        migrations.AddField(
            model_name='secret',
            name='allowed_groups',
            field=models.ManyToManyField(blank=True, related_name='allowed_passwords', through='secrets.SharedSecretData', to='auth.group'),
        ),
        migrations.AddField(
            model_name='secret',
            name='allowed_users',
            field=models.ManyToManyField(blank=True, related_name='allowed_passwords', through='secrets.SharedSecretData', to=settings.AUTH_USER_MODEL),
        ),
    ]



from datetime import timedelta

import logging

from ..audit.auditlog import log
from ..audit.models import LogEntry, AuditLogCategoryChoices
from ..secrets.models import SharedSecretData

from django.conf import settings
from django.contrib.auth.models import Group
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _
from huey import crontab
from huey.contrib.djhuey import periodic_task


huey_log = logging.getLogger('huey')


@periodic_task(crontab(**settings.HUEY_TASKS['scheduler_frequency']))
def prune_expired_shares():
    for share in SharedSecretData.objects.with_expiry_state().filter(is_expired=True):
        huey_log.info(
            _("Removing expired share of '{secret}' ({secret_id}) for {share_type} '{who}', was valid until {until}").format(
                secret=share.secret,
                secret_id=share.secret.hashid,
                share_type=_("user") if share.user else _("group"),
                until=share.granted_until,
                who=share.user or share.group,
            ),
        )
        share.delete()


@periodic_task(crontab(**settings.HUEY_TASKS['scheduler_frequency']))
def revoke_unused_shares():
    if settings.HUEY_TASKS['revoke_unused_shares_after_days'] is None:
        return

    grace_period = now() - timedelta(days=settings.HUEY_TASKS['revoke_unused_shares_after_days'])

    for share in SharedSecretData.objects.filter(
        granted_on__lt=grace_period,
    ):
        users_to_check = []

        if share.user:
            users_to_check.append(share.user)
        else:
            users_to_check.extend(share.group.user_set.all())

        do_revoke = True
        for user in users_to_check:
            accessed = LogEntry.objects.filter(
                actor=user,
                secret=share.secret,
                time__gte=grace_period,
            )
            if accessed:
                do_revoke = False
                # Skip further unnecessary checks.
                break

        if do_revoke:
            log(
                _(
                    "Share for {share_type} '{who}' automatically revoked, "
                    "not used since {grace_period}"
                ).format(
                    grace_period=grace_period,
                    share_type=_("user") if share.user else _("group"),
                    who=share.user or share.group,
                ),
                category=AuditLogCategoryChoices.SHARE_AUTOMATICALLY_REVOKED,
                level='info',
                secret=share.secret,
            )
            share.delete()



from django.conf.urls import include
from django.urls import path

handler404 = 'teamvault.views.handler404'

urlpatterns = (
    path('api/', include('teamvault.apps.secrets.api.urls'), name='api'),
    path('api-auth/', include('rest_framework.urls', namespace='rest_framework')),
    path('audit', include('teamvault.apps.audit.urls'), name='audit'),
    path('', include('teamvault.apps.secrets.urls'), name='secrets'),
    path('', include('teamvault.apps.accounts.urls'), name='accounts'),
    path('', include('social_django.urls', namespace='social')),
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('auth', '0005_alter_user_last_login_null'),
        ('secrets', '0006_auto_20150124_1103'),
    ]

    operations = [
        migrations.CreateModel(
            name='LogEntry',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', primary_key=True, auto_created=True)),
                ('message', models.TextField()),
                ('time', models.DateTimeField(auto_now_add=True)),
                ('actor', models.ForeignKey(null=True, on_delete=models.CASCADE, related_name='logged_actions', to=settings.AUTH_USER_MODEL, blank=True)),
                ('group', models.ForeignKey(null=True, on_delete=models.CASCADE, related_name='logged_actions', to='auth.Group', blank=True)),
                ('secret', models.ForeignKey(null=True, on_delete=models.CASCADE, related_name='logged_actions', to='secrets.Secret', blank=True)),
                ('secret_revision', models.ForeignKey(null=True, on_delete=models.CASCADE, related_name='logged_actions', to='secrets.SecretRevision', blank=True)),
                ('user', models.ForeignKey(null=True, on_delete=models.CASCADE, related_name='affected_by_actions', to=settings.AUTH_USER_MODEL, blank=True)),
            ],
            options={
                'ordering': ('-time',),
            },
        ),
    ]



from base64 import b64decode
from json import dumps

from django.contrib.auth.models import Group, User
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers
from rest_framework.exceptions import ValidationError
from rest_framework.reverse import reverse

from ..models import Secret, SecretRevision, SharedSecretData

ACCESS_POLICY_REPR = {
    Secret.ACCESS_POLICY_ANY: "any",
    Secret.ACCESS_POLICY_HIDDEN: "hidden",
    Secret.ACCESS_POLICY_DISCOVERABLE: "discoverable",
}
REPR_ACCESS_POLICY = {v: k for k, v in ACCESS_POLICY_REPR.items()}

CONTENT_TYPE_REPR = {
    Secret.CONTENT_CC: "cc",
    Secret.CONTENT_FILE: "file",
    Secret.CONTENT_PASSWORD: "password",
}
REPR_CONTENT_TYPE = {v: k for k, v in CONTENT_TYPE_REPR.items()}

SECRET_STATUS_REPR = {
    Secret.STATUS_DELETED: "deleted",
    Secret.STATUS_NEEDS_CHANGING: "needs_changing",
    Secret.STATUS_OK: "ok",
}
SECRET_REPR_STATUS = {v: k for k, v in SECRET_STATUS_REPR.items()}

REQUIRED_CC_FIELDS = {'holder', 'expiration_month', 'expiration_year', 'number', 'security_code'}


def _extract_data(validated_data):
    if 'password' in validated_data and not \
            REQUIRED_CC_FIELDS.intersection(set(validated_data.keys())):
        return (
            validated_data.pop('password'),
            Secret.CONTENT_PASSWORD,
        )
    elif 'file' in validated_data:
        return (
            b64decode(validated_data.pop('file').encode('ascii')),
            Secret.CONTENT_FILE,
        )
    elif REQUIRED_CC_FIELDS.intersection(set(validated_data.keys())):
        data = {}
        for cc_field in REQUIRED_CC_FIELDS:
            try:
                data[cc_field] = str(validated_data.pop(cc_field))
            except KeyError:
                raise serializers.ValidationError(
                    "missing required CC field '{}'".format(cc_field)
                )
        if 'password' in validated_data:
            data['password'] = validated_data.pop('password')
        else:
            data['password'] = ""
        return (
            dumps(data),
            Secret.CONTENT_CC,
        )
    else:
        return None, None


class SecretRevisionSerializer(serializers.HyperlinkedModelSerializer):
    api_url = serializers.HyperlinkedIdentityField(
        lookup_field='hashid',
        view_name='api.secret-revision_detail',
    )
    data_url = serializers.CharField(
        read_only=True,
        required=False,
        source='id',
    )
    set_by = serializers.SlugRelatedField(
        read_only=True,
        slug_field='username',
    )

    def to_representation(self, instance):
        rep = super(SecretRevisionSerializer, self).to_representation(instance)
        rep['data_url'] = reverse(
            'api.secret-revision_data',
            kwargs={'hashid': instance.hashid},
            request=self.context['request'],
        )
        return rep

    class Meta:
        model = SecretRevision
        fields = (
            'api_url',
            'created',
            'data_url',
            'set_by',
        )
        read_only_fields = (
            'created',
        )


class SecretSerializer(serializers.HyperlinkedModelSerializer):
    api_url = serializers.HyperlinkedIdentityField(
        lookup_field='hashid',
        view_name='api.secret_detail',
    )
    created_by = serializers.SlugRelatedField(
        default=serializers.CurrentUserDefault(),
        read_only=True,
        slug_field='username',
    )
    current_revision = serializers.HyperlinkedRelatedField(
        lookup_field='hashid',
        read_only=True,
        view_name='api.secret-revision_detail',
    )
    data_readable = serializers.BooleanField(
        read_only=True,
        required=False,
        source='id',
    )
    expiration_month = serializers.CharField(
        required=False,
        write_only=True,
    )
    expiration_year = serializers.CharField(
        required=False,
        write_only=True,
    )
    file = serializers.CharField(
        required=False,
        write_only=True,
    )
    holder = serializers.CharField(
        required=False,
        write_only=True,
    )
    number = serializers.CharField(
        required=False,
        write_only=True,
    )
    password = serializers.CharField(
        required=False,
        write_only=True,
    )
    security_code = serializers.CharField(
        required=False,
        write_only=True,
    )
    web_url = serializers.CharField(
        required=False,
        read_only=True,
    )

    def create(self, validated_data):
        data, content_type = _extract_data(validated_data)
        if not data:
            raise serializers.ValidationError("missing secret field (e.g. 'password')")

        instance = self.Meta.model.objects.create(**validated_data)
        instance.content_type = content_type
        instance._data = data
        instance.shared_users.add(instance.created_by)
        return instance

    def to_internal_value(self, data):
        try:
            data['access_policy'] = REPR_ACCESS_POLICY[data.get('access_policy', None)]
        except KeyError:
            # Validation will catch it
            pass

        try:
            data['status'] = SECRET_REPR_STATUS[data.get('status', None)]
        except KeyError:
            # Validation will catch it
            pass

        return super(SecretSerializer, self).to_internal_value(data)

    def to_representation(self, instance):
        rep = super(SecretSerializer, self).to_representation(instance)
        rep['access_policy'] = ACCESS_POLICY_REPR[rep['access_policy']]
        rep['content_type'] = CONTENT_TYPE_REPR[rep['content_type']]
        rep['data_readable'] = instance.is_readable_by_user(self.context['request'].user)
        rep['hashid'] = instance.hashid
        rep['status'] = SECRET_STATUS_REPR[rep['status']]
        rep['web_url'] = instance.full_url
        return rep

    def update(self, instance, validated_data):
        data, content_type = _extract_data(validated_data)
        if content_type and content_type != instance.content_type:
            raise serializers.ValidationError("wrong secret content type")
        if data:
            instance._data = data
        return instance

    def validate(self, data):
        if 'file' in data or 'filename' in data:
            if not ('file' in data and 'filename' in data):
                raise serializers.ValidationError("must include both file and filename")
        return data

    class Meta:
        model = Secret
        fields = (
            'access_policy',
            'api_url',
            'content_type',
            'created',
            'created_by',
            'current_revision',
            'data_readable',
            'description',
            'expiration_month',
            'expiration_year',
            'file',
            'filename',
            'holder',
            'last_read',
            'name',
            'needs_changing_on_leave',
            'number',
            'password',
            'security_code',
            'status',
            'url',
            'username',
            'web_url',
        )
        read_only_fields = (
            'content_type',
            'created',
            'last_read',
        )


class SharedSecretDataSerializer(serializers.ModelSerializer):
    granted_by = serializers.SlugRelatedField(
        default=serializers.CurrentUserDefault(),
        read_only=True,
        slug_field='username',
    )
    grant_description = serializers.CharField(
        allow_null=False,
    )
    granted_until = serializers.DateTimeField(
        allow_null=True,
    )
    group = serializers.SlugRelatedField(
        allow_null=True,
        queryset=Group.objects.all().order_by('name'),
        required=False,
        slug_field='name',
    )
    secret = serializers.SlugRelatedField(
        read_only=True,
        slug_field='hashid',
    )
    user = serializers.SlugRelatedField(
        allow_null=True,
        queryset=User.objects.exclude(is_active=False).order_by('username'),
        required=False,
        slug_field='username',
    )

    def validate(self, data):
        if (data['group'] and data['user']) or (not data['group'] and not data['user']):
            raise serializers.ValidationError('Choose exactly one group *or* one user to share the secret with.')

        secret = self.context['secret']
        if data['group']:
            entity_type = 'group'
            entity_name = data['group']
        else:
            entity_type = 'user'
            entity_name = data['user']

        if SharedSecretData.objects.filter(secret=secret, **{entity_type: entity_name}).exists():
            raise ValidationError(
                _(f'Secret {secret} was already shared with {entity_type} {entity_name}.').format(
                    secret=secret, entity_type=entity_type, entity_name=entity_name
                ),
                code='unique'
            )
        return data

    class Meta:
        model = SharedSecretData
        fields = ['id', 'grant_description', 'granted_on', 'granted_until', 'group', 'user', 'granted_by', 'secret']
        read_only_fields = (
            'id',
            'granted_by'
        )



from base64 import b32decode

from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _


def is_valid_b32_string(value, casefold=False):
    try:
        b32decode(value, casefold=casefold)
    except Exception:
        raise ValidationError(_('OTP key has wrong format. Please enter a valid OTP key.'))



# Generated by Django 4.2.1 on 2023-06-11 16:50

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("secrets", "0024_auto_20210824_1203"),
    ]

    operations = [
        migrations.AlterField(
            model_name="secret",
            name="description",
            field=models.TextField(
                blank=True, help_text="Further information on the secret.", null=True
            ),
        ),
        migrations.AlterField(
            model_name="secret",
            name="name",
            field=models.CharField(
                help_text="Enter a unique name for the secret", max_length=92
            ),
        ),
    ]



from django.urls import path

from .views import SecretDetail, SecretList, SecretRevisionDetail, SecretShare, SecretShareDetail, data_get, \
    generate_password_view, otp_get

urlpatterns = (
    path(
        'secrets/',
        SecretList.as_view(),
        name='api.secret_list',
    ),
    path(
        'secrets/<str:hashid>/',
        SecretDetail.as_view(),
        name='api.secret_detail',
    ),
    path(
        'secrets/<str:hashid>/shares/',
        SecretShare.as_view(),
        name='api.secret_share',
    ),
    path(
        'secrets/<str:hashid>/shares/<int:pk>',
        SecretShareDetail.as_view(),
        name='api.secret_share_detail',
    ),
    path(
        'secret-revisions/<str:hashid>/',
        SecretRevisionDetail.as_view(),
        name='api.secret-revision_detail',
    ),
    path(
        'secret-revisions/<str:hashid>/data',
        data_get,
        name='api.secret-revision_data',
    ),
    path(
        'secret-revisions/<str:hashid>/data/otp',
        otp_get,
        name='api.secret-revision_otp',
    ),
    path(
        'generate_password/',
        generate_password_view,
        name='api.generate-password',
    )
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations
from hashids import Hashids


def generate_hashids(apps, schema_editor):
    AccessRequest = apps.get_model("secrets", "AccessRequest")
    Secret = apps.get_model("secrets", "Secret")
    SecretRevision = apps.get_model("secrets", "SecretRevision")

    for model_class, hashid_namespace in (
        (AccessRequest, "AccessRequest"),
        (Secret, "Secret"),
        (SecretRevision, "SecretRevision"),
    ):
        for obj in model_class.objects.all():
            if not obj.hashid:
                # We cannot use the same salt for every model because
                # 1. sequentially create lots of secrets
                # 2. note the hashid of each secrets
                # 3. you can now enumerate access requests by using the same
                #    hashids
                # it's not a huge deal, but let's avoid it anyway
                hasher = Hashids(
                    min_length=settings.HASHID_MIN_LENGTH,
                    salt=hashid_namespace + settings.HASHID_SALT,
                )
                obj.hashid = hasher.encode(obj.pk)
            obj.save()


class Migration(migrations.Migration):

    dependencies = [
        ('secrets', '0008_auto_20150322_0944'),
    ]

    operations = [
        migrations.RunPython(generate_hashids),
    ]



from urllib.parse import urlencode

from django import template


register = template.Library()


@register.simple_tag()
def querystring(request, **kwargs):
    """
    Append or update params in a querystring.
    """
    querydict = request.GET.copy()
    for k, v in kwargs.items():
        if v is not None:
            querydict[k] = str(v)
        elif k in querydict:
            querydict.pop(k)
    qs = querydict.urlencode(safe='/')
    if qs:
        return '?' + qs
    else:
        return ''


@register.filter
def smart_pages(all_pages, current_page):
    all_pages = list(all_pages)
    smart_pages = set([
        1,
        all_pages[-1],
        current_page,
        max(min(current_page // 2, all_pages[-1]), 1),
        max(min(current_page + ((all_pages[-1] - current_page) // 2), all_pages[-1]), 1),
        max(min(current_page + 1, all_pages[-1]), 1),
        max(min(current_page + 2, all_pages[-1]), 1),
        max(min(current_page - 1, all_pages[-1]), 1),
        max(min(current_page - 2, all_pages[-1]), 1),
    ])
    return sorted(smart_pages)



from django.contrib.auth.models import User, Group
from django.db import IntegrityError
from django.test import TestCase

from teamvault.apps.secrets.models import Secret, SharedSecretData


class SecretConsistencyTestCase(TestCase):
    def setUp(self):
        User.objects.create(username='testuser')
        Group.objects.create(name='testgroup')

    def test_secret_unique_together(self):
        user = User.objects.get(username='testuser')
        group = Group.objects.get(name='testgroup')

        secret = Secret.objects.create(name="testsecret", created_by=user)
        secret.shared_users.add(user, through_defaults={})
        secret.shared_groups.add(group, through_defaults={})

        with self.assertRaises(IntegrityError):
            SharedSecretData.objects.create(secret=secret, user=user)
            SharedSecretData.objects.create(secret=secret, group=group)

    def test_shared_secret_data_only_one_constraint(self):
        user = User.objects.get(username='testuser')
        group = Group.objects.get(name='testgroup')
        secret = Secret.objects.create(name="testsecret", created_by=user)

        with self.assertRaises(IntegrityError):
            SharedSecretData.objects.create(secret=secret, group=group, user=user)



# Generated by Django 4.2.3 on 2023-09-05 11:57

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("auth", "0012_alter_user_first_name_max_length"),
        ("accounts", "0004_alter_usersettings_hide_deleted_secrets"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="UserSettings",
            new_name="UserProfile",
        ),
    ]



from django.contrib.auth.models import Group, User
from django.db import models
from django.utils.translation import gettext_lazy as _


class UserProfile(models.Model):
    # Since our static files are not served by some webserver but by TeamVault (/Whitenoise) directly
    # to keep the installation overhead low, we'd have to do the same thing with media files.
    # Static files will get replaced with each teamvault deployment, media files should not.
    # Because of that, we'd have to make admins configure a persistent directory for them.
    # For now, that trade-off is not worth it, so let's store avatars as binary data, instead.
    avatar = models.BinaryField(blank=True, null=True)
    default_sharing_groups = models.ManyToManyField(
        Group,
        blank=True,
        help_text=_('New secrets created by you will be shared with these groups.'),
        related_name='+',
    )
    hide_deleted_secrets = models.BooleanField(
        default=True,
        help_text=_('Hides deleted secrets per default. Enable them in filters to see them again.')
    )
    user = models.OneToOneField(User, on_delete=models.CASCADE, related_name='profile')



import django_filters

from teamvault.apps.secrets.models import Secret


class SecretFilter(django_filters.FilterSet):
    content_type = django_filters.MultipleChoiceFilter(choices=Secret.CONTENT_CHOICES)
    status = django_filters.MultipleChoiceFilter(choices=Secret.STATUS_CHOICES)

    class Meta:
        model = Secret
        fields = ['content_type', 'status']



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('secrets', '0002_secret_filename'),
    ]

    operations = [
        migrations.AddField(
            model_name='secretrevision',
            name='encrypted_data_sha256',
            field=models.CharField(default="0000000000000000000000000000000000000000000000000000000000000000", max_length=64),
            preserve_default=False,
        ),
        migrations.AlterUniqueTogether(
            name='secretrevision',
            unique_together=set([('encrypted_data_sha256', 'secret')]),
        ),
    ]



from django.urls import path

from . import views

urlpatterns = (
    path(
        '',
        views.dashboard,
        name='dashboard',
    ),
    path(
        'opensearch.xml',
        views.opensearch,
        name='opensearch',
    ),
    path(
        'secrets/',
        views.secret_list,
        name='secrets.secret-list',
    ),
    path(
        'secrets/<str:hashid>/',
        views.secret_detail,
        name='secrets.secret-detail',
    ),
    path(
        'secrets/<str:hashid>/delete',
        views.secret_delete,
        name='secrets.secret-delete',
    ),
    path(
        'secrets/<str:hashid>/download',
        views.secret_download,
        name='secrets.secret-download',
    ),
    path(
        'secrets/<str:hashid>/edit',
        views.secret_edit,
        name='secrets.secret-edit',
    ),
    path(
        'secrets/<str:hashid>/metadata',
        views.secret_metadata,
        name='secrets.secret-metadata',
    ),
    path(
        'secrets/<str:hashid>/restore',
        views.secret_restore,
        name='secrets.secret-restore',
    ),
    path(
        'secrets/<str:hashid>/share',
        views.secret_share_list,
        name='secrets.secret-share',
    ),
    path(
        'secrets/<str:hashid>/share/<int:share_id>/delete',
        views.secret_share_delete,
        name='secrets.secret-share-delete',
    ),
    path(
        'secrets/add/<str:content_type>',
        views.secret_add,
        name='secrets.secret-add',
    ),
    path(
        'secrets/live-search',
        views.secret_search,
        name='secrets.secret-search',
    ),
)



import secrets
import string
from urllib.parse import urlparse, parse_qs

from django.core.files.uploadhandler import MemoryFileUploadHandler, SkipFile

from .models import Secret


def extract_url_and_params(data):
    data_as_url = urlparse(data)
    data_params = parse_qs(data_as_url.query)
    return data_as_url, data_params


def serialize_add_edit_data(cleaned_data, secret):
    plaintext_data = {}
    if secret.content_type == Secret.CONTENT_PASSWORD:
        cleaned_data_as_url, data_params = extract_url_and_params(cleaned_data["otp_key_data"])
        if cleaned_data.get("password"):
            plaintext_data['password'] = cleaned_data['password']
        if data_params.get("secret"):
            plaintext_data["otp_key"] = data_params["secret"][0]
        if data_params.get("digits"):
            plaintext_data["digits"] = data_params["digits"][0]
        if data_params.get("algorithm"):
            plaintext_data["algorithm"] = data_params["algorithm"][0]
    elif secret.content_type == Secret.CONTENT_FILE:
        plaintext_data["file_content"] = cleaned_data['file'].read().decode("utf-8")
        secret.filename = cleaned_data['file'].name
        secret.save()
    elif secret.content_type == Secret.CONTENT_CC:
        plaintext_data = {
            'holder': cleaned_data['holder'],
            'number': cleaned_data['number'],
            'expiration_month': str(cleaned_data['expiration_month']),
            'expiration_year': str(cleaned_data['expiration_year']),
            'security_code': str(cleaned_data['security_code']),
            'password': cleaned_data['password'],
        }
    return plaintext_data


def generate_password(length, digits, upper, lower, special):
    characters = string.ascii_letters + string.digits + string.punctuation
    password = []
    password.extend(secrets.choice(string.digits) for _ in range(digits))
    password.extend(secrets.choice(string.ascii_lowercase) for _ in range(lower))
    password.extend(secrets.choice(string.ascii_uppercase) for _ in range(upper))
    password.extend(secrets.choice(string.punctuation) for _ in range(special))

    # Fill the rest of the lenght with random characters from all types
    password.extend(secrets.choice(characters) for _ in range(length - len(password)))

    # Randomly shuffle the characters, so they're not grouped by type
    secrets.SystemRandom().shuffle(password)

    return ''.join(password)


class CappedMemoryFileUploadHandler(MemoryFileUploadHandler):
    def receive_data_chunk(self, raw_data, start):
        if not self.activated:  # if the file size is too big, this handler will not be activated
            # if we use StopUpload here, forms will not get fully validated,
            # which leads to more form errors than we prefer
            # raise StopUpload(connection_reset=True)
            raise SkipFile()
        super(CappedMemoryFileUploadHandler, self).receive_data_chunk(raw_data, start)



from urllib.parse import quote, urlencode

from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import login_required, user_passes_test
from django.contrib.auth.models import Group, User
from django.core.exceptions import ObjectDoesNotExist, PermissionDenied
from django.http import Http404, HttpResponse, HttpResponseRedirect, JsonResponse
from django.shortcuts import get_object_or_404, redirect, render
from django.template.defaultfilters import pluralize
from django.urls import reverse
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _
from django.views.decorators.http import require_http_methods
from django.views.generic import DetailView, ListView, TemplateView
from django.views.generic.edit import CreateView, UpdateView
from django_htmx.http import trigger_client_event

from .filters import SecretFilter
from .forms import CCForm, FileForm, PasswordForm, SecretShareForm
from .models import AccessPermissionTypes, Secret, SharedSecretData
from .utils import serialize_add_edit_data
from ..accounts.models import UserProfile
from ..audit.auditlog import log
from ..audit.models import AuditLogCategoryChoices

CONTENT_TYPE_FORMS = {
    'cc': CCForm,
    'file': FileForm,
    'password': PasswordForm,
}
CONTENT_TYPE_IDS = {
    'cc': Secret.CONTENT_CC,
    'file': Secret.CONTENT_FILE,
    'password': Secret.CONTENT_PASSWORD,
}
CONTENT_TYPE_IDENTIFIERS = {v: k for k, v in CONTENT_TYPE_IDS.items()}
_CONTENT_TYPES = dict(Secret.CONTENT_CHOICES)
CONTENT_TYPE_NAMES = {
    'cc': _CONTENT_TYPES[Secret.CONTENT_CC],
    'file': _CONTENT_TYPES[Secret.CONTENT_FILE],
    'password': _CONTENT_TYPES[Secret.CONTENT_PASSWORD],
}


class Dashboard(TemplateView):
    template_name = "secrets/dashboard.html"

    def get_context_data(self, **kwargs):
        context = super(Dashboard, self).get_context_data(**kwargs)
        context['search_term'] = ""
        context['most_used_secrets'] = Secret.get_most_used_for_user(self.request.user, 10)
        context['readable_secrets'] = Secret.get_all_readable_by_user(self.request.user)
        context['recently_used_secrets'] = Secret.get_most_recently_used_for_user(self.request.user, 10)
        return context


dashboard = login_required(Dashboard.as_view())


class OpenSearch(TemplateView):
    content_type = "application/xml"
    template_name = "opensearch.xml"

    def get_context_data(self, **kwargs):
        context = super(OpenSearch, self).get_context_data(**kwargs)
        context['base_url'] = settings.BASE_URL
        return context


opensearch = OpenSearch.as_view()


class SecretAdd(CreateView):
    slug_field = 'hashid'
    slug_url_kwarg = 'hashid'

    def form_valid(self, form):
        secret = Secret()
        secret.content_type = CONTENT_TYPE_IDS[self.kwargs['content_type']]
        secret.created_by = self.request.user

        for attr in ('access_policy', 'description', 'name', 'needs_changing_on_leave', 'url', 'username'):
            if attr in form.cleaned_data:
                setattr(secret, attr, form.cleaned_data[attr])
        secret.save()
        plaintext_data = serialize_add_edit_data(form.cleaned_data, secret)
        secret.set_data(self.request.user, plaintext_data, skip_access_check=True)

        # Create share objects
        secret.share_data.create(user=self.request.user)
        if form.cleaned_data['access_policy'] != Secret.ACCESS_POLICY_ANY:
            try:
                secret.share_data.bulk_create(
                    [
                        SharedSecretData(
                            grant_description=form.cleaned_data['grant_description'],
                            granted_by=self.request.user,
                            group=group,
                            secret=secret,
                        )
                        for group in form.cleaned_data['shared_groups_on_create']
                    ]
                )
            except UserProfile.DoesNotExist:
                pass
        return HttpResponseRedirect(secret.get_absolute_url())

    def get_context_data(self, **kwargs):
        context = super(SecretAdd, self).get_context_data(**kwargs)
        try:
            context['pretty_content_type'] = CONTENT_TYPE_NAMES[self.kwargs['content_type']]
        except KeyError:
            raise Http404
        return context

    def get_initial(self):
        obj, _created = UserProfile.objects.get_or_create(user=self.request.user)
        return {'shared_groups_on_create': obj.default_sharing_groups.all()}

    def get_form_class(self):
        return CONTENT_TYPE_FORMS[self.kwargs['content_type']]

    def get_form(self, form_class=None):
        form = super().get_form(form_class=form_class)
        # handle files with sizes above FILE_UPLOAD_MAX_MEMORY_SIZE setting
        if (
                self.content_type == 'file'
                and self.request.method == 'POST'
                and not self.request.upload_handlers[0].activated
        ):
            form.add_error(
                f'file', f'File size too big. Allowed file size: {settings.FILE_UPLOAD_MAX_MEMORY_SIZE} bytes'
            )
        return form

    def get_template_names(self):
        return "secrets/addedit_content/{}.html".format(self.kwargs['content_type'])


secret_add = login_required(SecretAdd.as_view())


class SecretEdit(UpdateView):
    context_object_name = 'secret'
    slug_field = 'hashid'
    slug_url_kwarg = 'hashid'

    def form_valid(self, form):
        secret = self.object

        for attr in ('access_policy', 'description', 'name', 'needs_changing_on_leave', 'url', 'username'):
            if attr in form.cleaned_data:
                setattr(secret, attr, form.cleaned_data[attr])
        secret.save()
        plaintext_data = serialize_add_edit_data(form.cleaned_data, secret)
        if plaintext_data is not None:
            secret.set_data(self.request.user, plaintext_data)

        # clear saved otp key data cache after change
        if 'otp_key_data' in form.changed_data and form.cleaned_data.get('otp_key_data') and 'otp_key_data' in self.request.session:
            del self.request.session['otp_key_data']
        return HttpResponseRedirect(secret.get_absolute_url())

    def get_context_data(self, **kwargs):
        context = super(SecretEdit, self).get_context_data(**kwargs)
        context['current_revision'] = self.object.current_revision
        context['pretty_content_type'] = self.object.get_content_type_display()
        return context

    def get_form_class(self):
        return CONTENT_TYPE_FORMS[CONTENT_TYPE_IDENTIFIERS[self.object.content_type]]

    def get_form(self, form_class=None):
        form = super().get_form(form_class=form_class)
        # handle files with sizes above FILE_UPLOAD_MAX_MEMORY_SIZE setting
        if (
                self.content_type == 'file'
                and self.request.method == 'POST'
                and not self.request.upload_handlers[0].activated
        ):
            form.add_error(
                f'file', f'File size too big. Allowed file size: {settings.FILE_UPLOAD_MAX_MEMORY_SIZE} bytes'
            )
        return form

    def get_initial(self):
        if self.object.content_type == Secret.CONTENT_CC:
            data = self.object.get_data(self.request.user)
            return {
                'holder': data['holder'],
                'number': data['number'],
                'expiration_month': data['expiration_month'],
                'expiration_year': data['expiration_year'],
                'security_code': data['security_code'],
                'password': data['password'],
            }
        else:
            return {}

    def get_object(self, queryset=None):
        secret = get_object_or_404(Secret, hashid=self.kwargs['hashid'])
        secret.check_read_access(self.request.user)
        return secret

    def get_template_names(self):
        return "secrets/addedit_content/{}.html".format(CONTENT_TYPE_IDENTIFIERS[self.object.content_type])


secret_edit = login_required(SecretEdit.as_view())


@login_required
def secret_delete(request, hashid):
    secret = get_object_or_404(Secret, hashid=hashid)
    secret.check_read_access(request.user)
    if request.method == 'POST':
        log(_(
            "{user} deleted '{name}' ({id}:{revision})"
        ).format(
            id=secret.id,
            name=secret.name,
            revision=secret.current_revision.id,
            user=request.user.username,
        ),
            actor=request.user,
            category=AuditLogCategoryChoices.SECRET_CHANGED,
            level='info',
            secret=secret,
            secret_revision=secret.current_revision,
        )
        secret.status = Secret.STATUS_DELETED
        secret.save()
        messages.success(request, _('Successfully deleted secret'))
        return HttpResponseRedirect(
            reverse('secrets.secret-list') + "?" + urlencode([("search", secret.name.encode('utf-8'))]))
    else:
        return render(request, "secrets/secret_delete.html", {'secret': secret})


@login_required
@user_passes_test(lambda u: u.is_superuser)
def secret_restore(request, hashid):
    secret = get_object_or_404(Secret, hashid=hashid)
    secret.check_share_access(request.user)
    if request.method == 'POST':
        log(_(
            "{user} restored '{name}' ({id}:{revision})"
        ).format(
            id=secret.id,
            name=secret.name,
            revision=secret.current_revision.id,
            user=request.user.username,
        ),
            actor=request.user,
            category=AuditLogCategoryChoices.SECRET_CHANGED,
            level='info',
            secret=secret,
            secret_revision=secret.current_revision,
        )
        secret.status = Secret.STATUS_OK
        secret.save()
        messages.success(request, _('Successfully restored secret'))
        return redirect(secret.get_absolute_url())
    else:
        return render(request, "secrets/secret_restore.html", {'secret': secret})


@login_required
@require_http_methods(["GET"])
def secret_download(request, hashid):
    secret = get_object_or_404(Secret, hashid=hashid)
    if secret.content_type != Secret.CONTENT_FILE:
        raise Http404
    secret.check_read_access(request.user)

    response = HttpResponse(secret.get_data(request.user))
    response['Content-Disposition'] = \
        "attachment; filename*=UTF-8''{}".format(quote(secret.filename))
    response['Content-Type'] = "application/force-download"
    return response


class SecretDetail(DetailView):
    context_object_name = 'secret'
    model = Secret
    slug_field = 'hashid'
    slug_url_kwarg = 'hashid'

    def get_context_data(self, **kwargs):
        context = super(SecretDetail, self).get_context_data(**kwargs)
        secret = self.get_object()
        context['content_type'] = CONTENT_TYPE_IDENTIFIERS[secret.content_type]
        context['secret_revision'] = secret.current_revision
        context['readable'] = secret.is_readable_by_user(self.request.user)
        context['shareable'] = secret.is_shareable_by_user(self.request.user)
        context['secret_deleted'] = True if secret.status == Secret.STATUS_DELETED else False
        context['secret_url'] = reverse(
            'api.secret-revision_data',
            kwargs={'hashid': secret.current_revision.hashid},
        )

        context['show_password_update_alert'] = False
        if context['readable']:
            context['placeholder'] = secret.current_revision.length * "•"
            if context['readable'] == AccessPermissionTypes.SUPERUSER_ALLOWED:
                context['su_access'] = True
            if secret.status == Secret.STATUS_NEEDS_CHANGING and settings.PASSWORD_UPDATE_ALERT_ACTIVATED:
                context['show_password_update_alert'] = True
        return context

    def get_object(self, queryset=None):
        object = super(SecretDetail, self).get_object()
        if not object.is_visible_to_user(self.request.user):
            raise Http404
        return object

    def get_template_names(self):
        content_type = CONTENT_TYPE_IDENTIFIERS[self.object.content_type]
        return f'secrets/detail_content/{content_type}.html'


secret_detail = login_required(SecretDetail.as_view())


@login_required
@require_http_methods(["GET"])
def secret_metadata(request, hashid):
    secret = get_object_or_404(Secret, hashid=hashid)
    share_data = secret.share_data.with_expiry_state().filter(is_expired=False)
    context = {
        'allowed_groups': share_data.groups(),
        'allowed_users': share_data.users(),
        'secret': secret,
    }
    return render(request, context=context, template_name='secrets/detail_content/meta.html')


class SecretList(ListView):
    context_object_name = 'secrets'
    filter = None
    paginate_by = 25
    template_name = "secrets/secret_list.html"

    def get_context_data(self, **kwargs):
        context = super(SecretList, self).get_context_data(**kwargs)
        context['filter'] = self.filter
        context['readable_secrets'] = Secret.get_all_readable_by_user(self.request.user)
        return context

    def get_queryset(self):
        if "search" in self.request.GET:
            queryset = Secret.get_search_results(self.request.user, self.request.GET['search'])
        else:
            queryset = Secret.get_all_visible_to_user(self.request.user)

        try:
            if '3' not in self.request.GET.get('status', []) and self.request.user.profile.hide_deleted_secrets:
                queryset = queryset.exclude(status=Secret.STATUS_DELETED)
        except ObjectDoesNotExist:
            pass

        self.filter = SecretFilter(self.request.GET, queryset)
        return self.filter.qs


secret_list = login_required(SecretList.as_view())


class SecretShareList(CreateView):
    form_class = SecretShareForm
    slug_field = 'secret__hashid'
    slug_url_kwarg = 'hashid'
    template_name = 'secrets/share_content/share_list_modal.html'

    @cached_property
    def group_shares(self):
        if not self.queryset:
            self.queryset = self.get_queryset().with_expiry_state()
        return self.queryset.groups()

    @cached_property
    def user_shares(self):
        if not self.queryset:
            self.queryset = self.get_queryset().with_expiry_state()
        return self.queryset.users()

    def get_queryset(self):
        return SharedSecretData.objects.filter(
            secret__hashid=self.kwargs[self.slug_url_kwarg]
        ).prefetch_related('secret', 'user', 'group')

    def get_context_data(self, *, object_list=None, **kwargs):
        secret = get_object_or_404(Secret, hashid=self.kwargs[self.slug_url_kwarg])

        context = {
            'secret': secret,
            'shareable': secret.is_shareable_by_user(self.request.user),
            'shares': {
                'groups': self.group_shares,
                'users': self.user_shares,
            },
        }
        return super().get_context_data(**context)

    def form_valid(self, form):
        secret = Secret.objects.get(hashid=self.kwargs[self.slug_url_kwarg])
        permission = secret.is_shareable_by_user(self.request.user)
        if not permission:
            raise PermissionDenied()

        form_obj = form.save(commit=False)
        obj = secret.share(
            grant_description=form_obj.grant_description,
            granted_by=self.request.user,
            granted_until=form_obj.granted_until,
            group=form_obj.group,
            user=form_obj.user,
        )

        messages.success(self.request, _('Shared secret with {}'.format(obj.shared_entity_name)))

        # Clear cache
        delattr(self, 'group_shares')
        delattr(self, 'user_shares')

        context = self.get_context_data()
        context.update({
            'form': self.get_form_class()(),  # create a new blank form
            'show_object': {
                'id': obj.shared_entity.id,
                'type': 'group' if form.cleaned_data['group'] else 'user',
            }
        })
        response = self.render_to_response(context=context)
        trigger_client_event(response, 'refreshMetadata')
        return response

    def get_form_class(self):
        form_class = super(SecretShareList, self).get_form_class()

        # Exclude groups and users which the secret was already shared with
        form_class.base_fields['group'].queryset = Group.objects.all().exclude(
            name__in=self.group_shares.values_list('group__name', flat=True)
        ).order_by('name')
        form_class.base_fields['user'].queryset = User.objects.filter(is_active=True).order_by('username').exclude(
            username__in=self.user_shares.values_list('user__username', flat=True)
        ).order_by('username')
        return form_class


secret_share_list = login_required(SecretShareList.as_view())


@login_required
@require_http_methods(['DELETE'])
def secret_share_delete(request, hashid, share_id):
    share_data = get_object_or_404(SharedSecretData, secret__hashid=hashid, id=share_id)
    permission = share_data.secret.is_shareable_by_user(request.user)
    if not permission:
        raise PermissionDenied()

    secret = share_data.secret
    entity_type = share_data.shared_entity_type
    entity_name = share_data.shared_entity_name
    share_data.delete()

    messages.success(
        request,
        _('Successfully removed {} from allowed {}'.format(
            share_data.shared_entity_name, pluralize(share_data.shared_entity_type)
        ))
    )
    log(
        _("{user} removed access of {shared_entity_type} '{name}'").format(
            shared_entity_type=entity_type,
            name=entity_name,
            user=request.user.username,
        ),
        actor=request.user,
        category=(
            AuditLogCategoryChoices.SECRET_SUPERUSER_SHARE_REMOVED
            if permission == AccessPermissionTypes.SUPERUSER_ALLOWED
            else AuditLogCategoryChoices.SECRET_SHARE_REMOVED
        ),
        level='warning',
        secret=secret,
    )
    response = HttpResponse(status=200)
    trigger_client_event(response, 'refreshMetadata')
    trigger_client_event(response, 'refreshShareData')
    return response


@login_required
@require_http_methods(["GET"])
def secret_search(request):
    search_term = request.GET['q']
    search_limit = request.GET.get('limit', 15)
    search_results = []
    raw_results = Secret.get_search_results(request.user, search_term)
    filtered_secrets = list(raw_results[:search_limit])
    unreadable_secrets = filtered_secrets[:]
    sorted_secrets = []

    # sort readable passwords to top...
    for secret in filtered_secrets:
        metadata = ''
        icon = "lock-open"
        if secret.is_readable_by_user(request.user):
            if secret.content_type == secret.CONTENT_PASSWORD:
                icon = "user"
                metadata = getattr(secret, 'username')
            elif secret.content_type == secret.CONTENT_FILE:
                icon = "file"
                metadata = getattr(secret, 'filename')
            elif secret.content_type == secret.CONTENT_CC:
                icon = "credit-card"
                metadata = getattr(secret, 'description')
            sorted_secrets.append((secret, icon, metadata))
            unreadable_secrets.remove(secret)

    # and others to the bottom
    for secret in unreadable_secrets:
        sorted_secrets.append((secret, "lock", ''))

    for secret, icon, metadata in sorted_secrets:
        search_results.append({
            'icon': icon,
            'meta': metadata,
            'name': secret.name,
            'locked': True if icon == 'lock' else False,
            'hashid': secret.hashid,
            'url': reverse('secrets.secret-detail', kwargs={'hashid': secret.hashid}),
        })
    return JsonResponse({'count': raw_results.count(), 'results': search_results})
